@@ -1,14 +1,14 @@
-from gettext import gettext as _
-from logging import getLogger
-from urllib import parse
 import aiohttp
 import asyncio
 import json
 import re
 
 from aiohttp.client_exceptions import ClientResponseError
+from logging import getLogger
+from multidict import MultiDict
+from urllib import parse
 
-from pulpcore.plugin.download import HttpDownloader
+from pulpcore.plugin.download import DownloaderFactory, HttpDownloader
 
 
 log = getLogger(__name__)
@@ -78,12 +78,12 @@ class RegistryAuthHttpDownloader(HttpDownloader):
 
                             self.registry_auth["bearer"] = None
                             await self.update_token(response_auth_header, this_token, repo_name)
-                        return await self._run(handle_401=False)
+                        return await self._run(handle_401=False, extra_data=extra_data)
                     elif "Basic" in response_auth_header:
                         if self.remote.username:
                             basic = aiohttp.BasicAuth(self.remote.username, self.remote.password)
                             self.registry_auth["basic"] = basic.encode()
-                        return await self._run(handle_401=False)
+                        return await self._run(handle_401=False, extra_data=extra_data)
                 else:
                     raise
             to_return = await self._handle_response(response)
@@ -116,7 +116,7 @@ class RegistryAuthHttpDownloader(HttpDownloader):
             try:
                 token_base_url = auth_query_dict.pop("realm")
             except KeyError:
-                raise IOError(_("No realm specified for token auth challenge."))
+                raise IOError("No realm specified for token auth challenge.")
 
             # self defense strategy in cases when registry does not provide the scope
             if "scope" not in auth_query_dict:
@@ -163,3 +163,84 @@ class RegistryAuthHttpDownloader(HttpDownloader):
         elif basic_auth is not None:
             return {"Authorization": basic_auth}
         return {}
+
+
+class NoAuthSignatureDownloader(HttpDownloader):
+    """A downloader class suited for signature downloads."""
+
+    def raise_for_status(self, response):
+        """Log an error only if the status code of the response is not equal to 404.
+
+        Status codes equal to 404 signify that a signature could not be found on the server. This
+        case is still valid because it is not possible to determine the number of signatures
+        beforehand.
+        """
+        if response.status == 404:
+            raise FileNotFoundError()
+        else:
+            response.raise_for_status()
+
+
+class NoAuthDownloaderFactory(DownloaderFactory):
+    """
+    A downloader factory without any preset auth configuration, TLS or basic auth.
+    """
+
+    def _make_aiohttp_session_from_remote(self):
+        """
+        Same as DownloaderFactory._make_aiohttp_session_from_remote, excluding TLS configuration.
+
+        Returns:
+            :class:`aiohttp.ClientSession`
+
+        """
+        tcp_conn_opts = {"force_close": True}
+
+        headers = MultiDict({"User-Agent": NoAuthDownloaderFactory.user_agent()})
+        if self._remote.headers is not None:
+            for header_dict in self._remote.headers:
+                user_agent_header = header_dict.pop("User-Agent", None)
+                if user_agent_header:
+                    headers["User-Agent"] = f"{headers['User-Agent']}, {user_agent_header}"
+                headers.extend(header_dict)
+
+        conn = aiohttp.TCPConnector(**tcp_conn_opts)
+        total = self._remote.total_timeout
+        sock_connect = self._remote.sock_connect_timeout
+        sock_read = self._remote.sock_read_timeout
+        connect = self._remote.connect_timeout
+
+        timeout = aiohttp.ClientTimeout(
+            total=total, sock_connect=sock_connect, sock_read=sock_read, connect=connect
+        )
+        return aiohttp.ClientSession(
+            connector=conn, timeout=timeout, headers=headers, requote_redirect_url=False
+        )
+
+    def _http_or_https(self, download_class, url, **kwargs):
+        """
+        Same as DownloaderFactory._http_or_https, excluding the basic auth credentials.
+
+        Args:
+            download_class (:class:`~pulpcore.plugin.download.BaseDownloader`): The download
+                class to be instantiated.
+            url (str): The download URL.
+            kwargs (dict): All kwargs are passed along to the downloader. At a minimum, these
+                include the :class:`~pulpcore.plugin.download.BaseDownloader` parameters.
+
+        Returns:
+            :class:`~pulpcore.plugin.download.HttpDownloader`: A downloader that
+            is configured with the remote settings.
+
+        """
+        options = {"session": self._session}
+        if self._remote.proxy_url:
+            options["proxy"] = self._remote.proxy_url
+            if self._remote.proxy_username and self._remote.proxy_password:
+                options["proxy_auth"] = aiohttp.BasicAuth(
+                    login=self._remote.proxy_username, password=self._remote.proxy_password
+                )
+
+        kwargs["throttler"] = self._remote.download_throttler if self._remote.rate_limit else None
+
+        return download_class(url, **options, **kwargs)
