@@ -1,6 +1,5 @@
 import logging
 import os
-from gettext import gettext as _
 
 from asgiref.sync import sync_to_async
 
@@ -11,6 +10,9 @@ from multidict import MultiDict
 
 from pulpcore.plugin.content import Handler, PathNotResolved
 from pulpcore.plugin.models import ContentArtifact
+from pulpcore.plugin.content import ArtifactResponse
+
+from pulp_container.app.cache import RegistryContentCache
 from pulp_container.app.models import ContainerDistribution, Tag, Blob
 from pulp_container.app.schema_convert import Schema2toSchema1ConverterWrapper
 from pulp_container.app.utils import get_accepted_media_types
@@ -45,14 +47,14 @@ class Registry(Handler):
         return [path]
 
     @staticmethod
-    async def _dispatch(file, headers):
+    async def _dispatch(artifact, headers):
         """
         Stream a file back to the client.
 
         Stream the bits.
 
         Args:
-            file (:class:`django.db.models.fields.files.FieldFile`): File to respond with
+            artifact (:class:`pulpcore.app.models.Artifact`): Artifact to respond with
             headers (dict): A dictionary of response headers.
 
         Raises:
@@ -60,21 +62,30 @@ class Registry(Handler):
             NotImplementedError: If file is stored in a file storage we can't handle
 
         Returns:
-            The :class:`aiohttp.web.FileResponse` for the file.
+            The :class:`aiohttp.web.StreamedResponse` for the Artifact.
 
         """
-        path = os.path.join(settings.MEDIA_ROOT, file.name)
-        if not os.path.exists(path):
-            raise Exception(_("Expected path '{}' is not found").format(path))
-
         full_headers = MultiDict()
 
         full_headers["Content-Type"] = headers["Content-Type"]
         full_headers["Docker-Content-Digest"] = headers["Docker-Content-Digest"]
         full_headers["Docker-Distribution-API-Version"] = "registry/2.0"
-        file_response = web.FileResponse(path, headers=full_headers)
-        return file_response
 
+        if settings.DEFAULT_FILE_STORAGE == "pulpcore.app.models.storage.FileSystem":
+            file = artifact.file
+            path = os.path.join(settings.MEDIA_ROOT, file.name)
+            if not os.path.exists(path):
+                raise Exception("Expected path '{}' is not found".format(path))
+            return web.FileResponse(path, headers=full_headers)
+        elif not settings.REDIRECT_TO_OBJECT_STORAGE:
+            return ArtifactResponse(artifact=artifact, headers=headers)
+        else:
+            raise NotImplementedError("Redirecting to this storage is not implemented.")
+
+    @RegistryContentCache(
+        base_key=lambda req, cac: Registry.find_base_path_cached(req, cac),
+        auth=lambda req, cac, bk: Registry.auth_cached(req, cac, bk),
+    )
     async def get_tag(self, request):
         """
         Match the path and stream either Manifest or ManifestList.
@@ -91,13 +102,14 @@ class Registry(Handler):
                 streamed back to the client.
 
         """
-        self._reset_db_connection()
 
         path = request.match_info["path"]
         tag_name = request.match_info["tag_name"]
         distribution = await sync_to_async(self._match_distribution)(path)
         await sync_to_async(self._permit)(request, distribution)
         repository_version = await sync_to_async(distribution.get_repository_version)()
+        if not repository_version:
+            raise PathNotResolved(tag_name)
         accepted_media_types = get_accepted_media_types(request.headers)
 
         try:
@@ -114,7 +126,7 @@ class Registry(Handler):
             and tag.tagged_manifest.media_type not in accepted_media_types
         ):
             log.warn(
-                _("OCI format found, but the client only accepts {accepted_media_types}.").format(
+                "OCI format found, but the client only accepts {accepted_media_types}.".format(
                     accepted_media_types=accepted_media_types
                 )
             )
@@ -163,7 +175,7 @@ class Registry(Handler):
             ca = await sync_to_async(lambda x: x[0])(tag.tagged_manifest.contentartifact_set.all())
             return await self._stream_content_artifact(request, web.StreamResponse(), ca)
         else:
-            return await Registry._dispatch(artifact.file, response_headers)
+            return await Registry._dispatch(artifact, response_headers)
 
     @staticmethod
     async def dispatch_converted_schema(tag, accepted_media_types, path):
@@ -199,17 +211,22 @@ class Registry(Handler):
         }
         return web.Response(text=result.text, headers=response_headers)
 
+    @RegistryContentCache(
+        base_key=lambda req, cac: Registry.find_base_path_cached(req, cac),
+        auth=lambda req, cac, bk: Registry.auth_cached(req, cac, bk),
+    )
     async def get_by_digest(self, request):
         """
         Return a response to the "GET" action.
         """
-        self._reset_db_connection()
 
         path = request.match_info["path"]
         digest = "sha256:{digest}".format(digest=request.match_info["digest"])
         distribution = await sync_to_async(self._match_distribution)(path)
         await sync_to_async(self._permit)(request, distribution)
         repository_version = await sync_to_async(distribution.get_repository_version)()
+        if not repository_version:
+            raise PathNotResolved(path)
         if digest == EMPTY_BLOB:
             return await Registry._empty_blob()
         try:
@@ -233,7 +250,7 @@ class Registry(Handler):
         else:
             artifact = ca.artifact
             if artifact:
-                return await Registry._dispatch(artifact.file, headers)
+                return await Registry._dispatch(artifact, headers)
             else:
                 return await self._stream_content_artifact(request, web.StreamResponse(), ca)
 
@@ -245,7 +262,7 @@ class Registry(Handler):
             255, 46, 175, 181, 239, 0, 4, 0, 0,
         ]
         # fmt: on
-        body = bytearray(empty_tar)
+        body = bytes(empty_tar)
         response_headers = {
             "Docker-Content-Digest": EMPTY_BLOB,
             "Content-Type": BLOB_CONTENT_TYPE,
