@@ -4,13 +4,16 @@ Check `Plugin Writer's Guide`_ for more details.
 . _Plugin Writer's Guide:
     http://docs.pulpproject.org/plugins/plugin-writer/index.html
 """
+import base64
+import binascii
 import json
 import logging
 import hashlib
 import re
-from collections import namedtuple
 import time
 
+from collections import namedtuple
+
 from urllib.parse import urlparse, urlunparse, parse_qs, urlencode
 from tempfile import NamedTemporaryFile
 
@@ -21,16 +24,15 @@ from django.shortcuts import get_object_or_404
 
 from django.conf import settings
 
-from guardian.shortcuts import get_objects_for_user
-
 from pulpcore.plugin.models import Artifact, ContentArtifact, Task, UploadChunk
 from pulpcore.plugin.files import PulpTemporaryUploadedFile
 from pulpcore.plugin.tasking import add_and_remove, dispatch
+from pulpcore.plugin.util import get_objects_for_user
+from rest_framework.authentication import BasicAuthentication
 from rest_framework.exceptions import (
     AuthenticationFailed,
     NotAuthenticated,
     PermissionDenied,
-    NotFound,
     ParseError,
     Throttled,
     ValidationError,
@@ -48,6 +50,17 @@ from rest_framework.views import APIView
 from pulp_container.app import models, serializers
 from pulp_container.app.access_policy import RegistryAccessPolicy
 from pulp_container.app.authorization import AuthorizationService
+from pulp_container.app.cache import find_base_path_cached, RegistryApiCache
+from pulp_container.app.exceptions import (
+    InvalidRequest,
+    RepositoryNotFound,
+    RepositoryInvalid,
+    BlobNotFound,
+    BlobInvalid,
+    ManifestNotFound,
+    ManifestInvalid,
+    ManifestSignatureInvalid,
+)
 from pulp_container.app.redirects import (
     FileStorageRedirects,
     S3StorageRedirects,
@@ -59,116 +72,25 @@ from pulp_container.app.token_verification import (
     RegistryPermission,
     TokenPermission,
 )
-from pulp_container.constants import EMPTY_BLOB
+from pulp_container.app.utils import (
+    determine_media_type,
+    extract_data_from_signature,
+    has_task_completed,
+    validate_manifest,
+)
+from pulp_container.constants import (
+    EMPTY_BLOB,
+    SIGNATURE_API_EXTENSION_VERSION,
+    SIGNATURE_HEADER,
+    SIGNATURE_PAYLOAD_MAX_SIZE,
+    SIGNATURE_TYPE,
+)
 
 FakeView = namedtuple("FakeView", ["action", "get_object"])
 
 log = logging.getLogger(__name__)
 
 
-class RepositoryNotFound(NotFound):
-    """Exception to render a 404 with the code 'NAME_UNKNOWN'"""
-
-    def __init__(self, name):
-        """Initialize the exception with the repository name."""
-        super().__init__(
-            detail={
-                "errors": [
-                    {
-                        "code": "NAME_UNKNOWN",
-                        "message": "Repository not found.",
-                        "detail": {"name": name},
-                    }
-                ]
-            }
-        )
-
-
-class RepositoryInvalid(ParseError):
-    """Exception to render a 400 with the code 'NAME_INVALID'"""
-
-    def __init__(self, name, message=None):
-        """Initialize the exception with the repository name."""
-        message = message or "Invalid repository name."
-        super().__init__(
-            detail={
-                "errors": [{"code": "NAME_INVALID", "message": message, "detail": {"name": name}}]
-            }
-        )
-
-
-class BlobNotFound(NotFound):
-    """Exception to render a 404 with the code 'BLOB_UNKNOWN'"""
-
-    def __init__(self, digest):
-        """Initialize the exception with the blob digest."""
-        super().__init__(
-            detail={
-                "errors": [
-                    {
-                        "code": "BLOB_UNKNOWN",
-                        "message": "Blob not found.",
-                        "detail": {"digest": digest},
-                    }
-                ]
-            }
-        )
-
-
-class BlobInvalid(ParseError):
-    """Exception to render a 400 with the code 'BLOB_UNKNOWN'"""
-
-    def __init__(self, digest):
-        """Initialize the exception with the blob digest."""
-        super().__init__(
-            detail={
-                "errors": [
-                    {
-                        "code": "BLOB_UNKNOWN",
-                        "message": "blob unknown to registry",
-                        "detail": {"digest": digest},
-                    }
-                ]
-            }
-        )
-
-
-class ManifestNotFound(NotFound):
-    """Exception to render a 404 with the code 'MANIFEST_UNKNOWN'"""
-
-    def __init__(self, reference):
-        """Initialize the exception with the manifest reference."""
-        super().__init__(
-            detail={
-                "errors": [
-                    {
-                        "code": "MANIFEST_UNKNOWN",
-                        "message": "Manifest not found.",
-                        "detail": {"reference": reference},
-                    }
-                ]
-            }
-        )
-
-
-class ManifestInvalid(ParseError):
-    """Exception to render a 400 with the code 'MANIFEST_INVALID'"""
-
-    def __init__(self, digest):
-        """Initialize the exception with the manifest digest."""
-        super().__init__(
-            detail={
-                "errors": [
-                    {
-                        "code": "MANIFEST_INVALID",
-                        "message": "manifest invalid",
-                        "detail": {"digest": digest},
-                    }
-                ]
-            }
-        )
-
-
 class ContentRenderer(BaseRenderer):
     """
     Rendered class for rendering Manifest and Blob responses.
@@ -234,6 +156,22 @@ class ManifestResponse(Response):
         super().__init__(headers=headers, status=status, content_type=manifest.media_type)
 
 
+class ManifestSignatureResponse(Response):
+    """
+    An HTTP response class after creating an image signature.
+    """
+
+    def __init__(self, signature, path, status=201):
+        """Initialize the headers with the path to the repository and corresponding digests."""
+        headers = {
+            "Location": "/extensions/v2/{path}/signatures/{digest}".format(
+                path=path, digest=signature.signed_manifest.digest
+            ),
+            "Content-Length": 0,
+        }
+        super().__init__(headers=headers, status=status)
+
+
 class BlobResponse(Response):
     """
     An HTTP response class for returning Blobs.
@@ -292,7 +230,7 @@ class ContainerRegistryApiMixin:
         Provide common headers to all responses.
         """
         headers = super().default_response_headers
-        headers.update({"Docker-Distribution-Api-Version": "registry/2.0"})
+        headers.update({"Docker-Distribution-Api-Version": "registry/2.0", SIGNATURE_HEADER: "1"})
         return headers
 
     def get_exception_handler_context(self):
@@ -418,6 +356,7 @@ class BearerTokenView(APIView):
     """
 
     # Allow everyone to access but still value authenticated users.
+    authentication_classes = [BasicAuthentication]
     permission_classes = []
 
     def get(self, request):
@@ -426,7 +365,7 @@ class BearerTokenView(APIView):
             service = request.query_params["service"]
         except KeyError:
             raise ParseError(detail="No service name provided.")
-        scope = request.query_params.get("scope", "")
+        scope = request.query_params.getlist("scope", [])
 
         authorization_service = AuthorizationService(self.request.user, service, scope)
         data = authorization_service.generate_token()
@@ -527,8 +466,7 @@ class CatalogView(ContainerRegistryApiMixin, ListAPIView):
             self.request.user, distribution_permission, queryset
         )
 
-        namespace_refs = queryset.values_list("namespace", flat=True)
-        namespaces = models.ContainerNamespace.objects.filter(pk__in=namespace_refs)
+        namespaces = models.ContainerNamespace.objects.all()
         repositories_by_namespace = get_objects_for_user(
             self.request.user, namespace_permission, namespaces
         )
@@ -617,47 +555,124 @@ class BlobUploads(ContainerRegistryApiMixin, ViewSet):
     model = models.Upload
     queryset = models.Upload.objects.all()
 
-    content_range_pattern = re.compile(r"^(?P<start>\d+)-(?P<end>\d+)$")
+    content_range_pattern = re.compile(r"^([0-9]+)-([0-9]+)$")
 
     def create(self, request, path):
         """
         Create a new upload.
+
+        Note: We do not support monolithic upload.
         """
         _, repository = self.get_dr_push(request, path, create=True)
 
-        upload = models.Upload(repository=repository, size=0)
-        upload.save()
-        response = UploadResponse(upload=upload, path=path, request=request)
+        if self.tries_to_mount_blob(request):
+            response = self.mount_blob(request, path, repository)
+        else:
+            upload = models.Upload(repository=repository, size=0)
+            upload.save()
+            response = UploadResponse(upload=upload, path=path, request=request)
 
         return response
 
+    @staticmethod
+    def tries_to_mount_blob(request):
+        """Check if a client is trying to perform cross repository blob mounting."""
+        return (request.query_params.keys()) == {"from", "mount"}
+
+    def mount_blob(self, request, path, repository):
+        """Mount a blob that is already present in another repository."""
+        from_path = request.query_params["from"]
+        try:
+            distribution = models.ContainerDistribution.objects.get(base_path=from_path)
+        except models.ContainerDistribution.DoesNotExist:
+            raise RepositoryNotFound(name=path)
+
+        try:
+            version = distribution.repository_version or distribution.repository.latest_version()
+        except AttributeError:
+            # the distribution does not contain reference to the source repository version
+            raise RepositoryNotFound(name=from_path)
+
+        digest = request.query_params["mount"]
+        try:
+            blob = models.Blob.objects.get(digest=digest, pk__in=version.content)
+        except models.Blob.DoesNotExist:
+            raise BlobNotFound(digest=digest)
+
+        dispatched_task = dispatch(
+            add_and_remove,
+            shared_resources=[version.repository],
+            exclusive_resources=[repository],
+            kwargs={
+                "repository_pk": str(repository.pk),
+                "add_content_units": [str(blob.pk)],
+                "remove_content_units": [],
+            },
+        )
+
+        # Wait a small amount of time
+        for dummy in range(3):
+            time.sleep(1)
+            task = Task.objects.get(pk=dispatched_task.pk)
+            if task.state == "completed":
+                task.delete()
+                return BlobResponse(blob, path, 201, request)
+            elif task.state in ["waiting", "running"]:
+                continue
+            else:
+                error = task.error
+                task.delete()
+                raise Exception(str(error))
+        raise Throttled()
+
     def partial_update(self, request, path, pk=None):
         """
         Process a chunk that will be appended to an existing upload.
         """
         _, repository = self.get_dr_push(request, path)
-        chunk = request.META["wsgi.input"]
-        if "Content-Range" in request.headers or "digest" not in request.query_params:
-            whole = False
-        else:
-            whole = True
-
-        if whole:
-            start = 0
-        else:
-            content_range = request.META.get("HTTP_CONTENT_RANGE", "")
-            match = self.content_range_pattern.match(content_range)
-            start = 0 if not match else int(match.group("start"))
-
         upload = get_object_or_404(models.Upload, repository=repository, pk=pk)
+        chunk = request.META["wsgi.input"]
+        if range_header := request.headers.get("Content-Range"):
+            found = self.content_range_pattern.match(range_header)
+            if not found:
+                raise InvalidRequest(message="Invalid range header")
+            start = int(found.group(1))
+            end = int(found.group(2))
+            length = end - start + 1
+
+        else:
+            length = int(request.headers.get("Content-Length", 0))
+            start = 0
 
-        chunk = ContentFile(chunk.read())
         with transaction.atomic():
             if upload.size != start:
                 raise Exception
 
-            upload.append(chunk, upload.size)
-            upload.size += chunk.size
+            # if more chunks
+            if range_header:
+                chunk = ContentFile(chunk.read())
+                upload.append(chunk, upload.size)
+            else:
+                # 1 chunk
+                # do not add to the upload, create artifact right away
+                with NamedTemporaryFile("ab") as temp_file:
+                    temp_file.write(chunk.read())
+                    temp_file.flush()
+
+                    uploaded_file = PulpTemporaryUploadedFile.from_file(
+                        File(open(temp_file.name, "rb"))
+                    )
+                try:
+                    artifact = Artifact.init_and_validate(uploaded_file)
+                    artifact.save()
+                except IntegrityError:
+                    artifact = Artifact.objects.get(sha256=artifact.sha256)
+                    artifact.touch()
+                upload.artifact = artifact
+                if not length:
+                    length = artifact.size
+
+            upload.size += length
             upload.save()
 
         return UploadResponse(upload=upload, path=path, request=request, status=204)
@@ -665,6 +680,8 @@ class BlobUploads(ContainerRegistryApiMixin, ViewSet):
     def put(self, request, path, pk=None):
         """
         Create a blob from uploaded chunks.
+
+        Note: We do not support monolithic upload.
         """
         _, repository = self.get_dr_push(request, path)
 
@@ -698,25 +715,34 @@ class BlobUploads(ContainerRegistryApiMixin, ViewSet):
                 task.delete()
                 raise Exception(str(error))
 
-        chunks = UploadChunk.objects.filter(upload=upload).order_by("offset")
+        if artifact := upload.artifact:
+            if artifact.sha256 != digest[len("sha256:") :]:
+                raise Exception("The digest did not match")
+            artifact.touch()
+        else:
+            chunks = UploadChunk.objects.filter(upload=upload).order_by("offset")
+            with NamedTemporaryFile("ab") as temp_file:
+                for chunk in chunks:
+                    temp_file.write(chunk.file.read())
+                    chunk.file.close()
+                temp_file.flush()
 
-        with NamedTemporaryFile("ab") as temp_file:
-            for chunk in chunks:
-                temp_file.write(chunk.file.read())
-                chunk.file.close()
-            temp_file.flush()
-
-            uploaded_file = PulpTemporaryUploadedFile.from_file(File(open(temp_file.name, "rb")))
-
-        if uploaded_file.hashers["sha256"].hexdigest() == digest[len("sha256:") :]:
+                uploaded_file = PulpTemporaryUploadedFile.from_file(
+                    File(open(temp_file.name, "rb"))
+                )
+            if uploaded_file.hashers["sha256"].hexdigest() != digest[len("sha256:") :]:
+                upload.delete()
+                raise Exception("The digest did not match")
             try:
                 artifact = Artifact.init_and_validate(uploaded_file)
                 artifact.save()
             except IntegrityError:
                 artifact = Artifact.objects.get(sha256=artifact.sha256)
                 artifact.touch()
+
+        with transaction.atomic():
             try:
-                blob = models.Blob(digest=digest, media_type=models.MEDIA_TYPE.REGULAR_BLOB)
+                blob = models.Blob(digest=digest)
                 blob.save()
             except IntegrityError:
                 blob = models.Blob.objects.get(digest=digest)
@@ -731,35 +757,21 @@ class BlobUploads(ContainerRegistryApiMixin, ViewSet):
                 if not ca.artifact:
                     ca.artifact = artifact
                     ca.save(update_fields=["artifact"])
-            upload.delete()
+        upload.delete()
 
-            dispatched_task = dispatch(
-                add_and_remove,
-                shared_resources=[f"upload:{pk}"],
-                exclusive_resources=[repository],
-                kwargs={
-                    "repository_pk": str(repository.pk),
-                    "add_content_units": [str(blob.pk)],
-                    "remove_content_units": [],
-                },
-            )
+        dispatched_task = dispatch(
+            add_and_remove,
+            shared_resources=[f"upload:{pk}"],
+            exclusive_resources=[repository],
+            kwargs={
+                "repository_pk": str(repository.pk),
+                "add_content_units": [str(blob.pk)],
+                "remove_content_units": [],
+            },
+        )
 
-            # Wait a small amount of time
-            for dummy in range(3):
-                time.sleep(1)
-                task = Task.objects.get(pk=dispatched_task.pk)
-                if task.state == "completed":
-                    task.delete()
-                    return BlobResponse(blob, path, 201, request)
-                elif task.state in ["waiting", "running"]:
-                    continue
-                else:
-                    error = task.error
-                    task.delete()
-                    raise Exception(str(error))
-            raise Throttled()
-        else:
-            raise Exception("The digest did not match")
+        if has_task_completed(dispatched_task):
+            return BlobResponse(blob, path, 201, request)
 
 
 class RedirectsMixin:
@@ -773,7 +785,10 @@ class RedirectsMixin:
         """
         super().__init__(*args, **kwargs)
 
-        if settings.DEFAULT_FILE_STORAGE == "pulpcore.app.models.storage.FileSystem":
+        if (
+            settings.DEFAULT_FILE_STORAGE == "pulpcore.app.models.storage.FileSystem"
+            or not settings.REDIRECT_TO_OBJECT_STORAGE
+        ):
             self.redirects_class = FileStorageRedirects
         elif settings.DEFAULT_FILE_STORAGE == "storages.backends.s3boto3.S3Boto3Storage":
             self.redirects_class = S3StorageRedirects
@@ -794,10 +809,17 @@ class Blobs(RedirectsMixin, ContainerRegistryApiMixin, ViewSet):
         """
         Responds to HEAD requests about blobs
         """
-        return self.get(request, path, pk=pk)
+        return self.handle_safe_method(request, path, pk=pk)
 
-    def get(self, request, path, pk=None):
-        """Handles GET requests for Blobs."""
+    def get(self, request, path, pk):
+        """
+        Responds to GET requests about blobs
+        """
+        return self.handle_safe_method(request, path, pk)
+
+    @RegistryApiCache(base_key=lambda req, cac: find_base_path_cached(req, cac))
+    def handle_safe_method(self, request, path, pk):
+        """Handles safe requests for Blobs."""
         distribution, _, repository_version = self.get_drv_pull(path)
         redirects = self.redirects_class(distribution, path, request)
 
@@ -823,13 +845,19 @@ class Manifests(RedirectsMixin, ContainerRegistryApiMixin, ViewSet):
         """
         Responds to HEAD requests about manifests by reference
         """
+        return self.handle_safe_method(request, path, pk=pk)
 
-        return self.get(request, path, pk=pk)
-
-    def get(self, request, path, pk=None):
+    def get(self, request, path, pk):
         """
         Responds to GET requests about manifests by reference
         """
+        return self.handle_safe_method(request, path, pk)
+
+    @RegistryApiCache(base_key=lambda req, cac: find_base_path_cached(req, cac))
+    def handle_safe_method(self, request, path, pk):
+        """
+        Responds to safe requests about manifests by reference
+        """
         distribution, _, repository_version = self.get_drv_pull(path)
         redirects = self.redirects_class(distribution, path, request)
 
@@ -848,40 +876,155 @@ class Manifests(RedirectsMixin, ContainerRegistryApiMixin, ViewSet):
         """
         Responds with the actual manifest
         """
-        _, repository = self.get_dr_push(request, path)
         # iterate over all the layers and create
         chunk = request.META["wsgi.input"]
         artifact = self.receive_artifact(chunk)
         manifest_digest = "sha256:{id}".format(id=artifact.sha256)
+
         with storage.open(artifact.file.name) as artifact_file:
             raw_data = artifact_file.read()
-        content_data = json.loads(raw_data)
-        # oci format might not contain mediaType in the manifest.json, docker should
-        # hence need to check request content type
-        if request.content_type not in (
-            models.MEDIA_TYPE.MANIFEST_V2,
-            models.MEDIA_TYPE.MANIFEST_OCI,
-        ):
-            # we suport only v2 docker/oci schema upload
-            raise ManifestInvalid(digest=manifest_digest)
-        # both docker/oci format should contain config, digest, mediaType, size
-        config_layer = content_data.get("config")
-        try:
-            config_digest = config_layer.get("digest")
-            config_blob = models.Blob.objects.get(digest=config_digest)
-        except models.Blob.DoesNotExist:
-            raise BlobNotFound(digest=config_digest)
-        config_media_type = config_layer.get("mediaType")
-        if config_media_type not in (
-            models.MEDIA_TYPE.CONFIG_BLOB,
-            models.MEDIA_TYPE.CONFIG_BLOB_OCI,
-        ):
-            raise BlobInvalid(digest=config_blob.digest)
 
+        content_data = json.loads(raw_data)
+
+        media_type = determine_media_type(content_data, request)
+        validate_manifest(content_data, media_type, manifest_digest)
+
+        # when a user uploads a manifest list with zero listed manifests (no blobs were uploaded
+        # before) and the specified repository has not been created yet, create the repository
+        # without raising an error
+        create_new_repo = media_type in (
+            models.MEDIA_TYPE.MANIFEST_LIST,
+            models.MEDIA_TYPE.INDEX_OCI,
+        )
+        _, repository = self.get_dr_push(request, path, create=create_new_repo)
+
+        if media_type in (
+            models.MEDIA_TYPE.MANIFEST_LIST,
+            models.MEDIA_TYPE.INDEX_OCI,
+        ):
+            manifests = {}
+            for manifest in content_data.get("manifests"):
+                manifests[manifest["digest"]] = manifest["platform"]
+
+            digests = set(manifests.keys())
+            found_manifests = models.Manifest.objects.filter(digest__in=digests)
+
+            if (len(manifests) - found_manifests.count()) != 0:
+                ManifestInvalid(digest=manifest_digest)
+
+            manifest_list = self._save_manifest(artifact, manifest_digest, media_type)
+
+            manifests_to_list = []
+            for manifest in found_manifests:
+                platform = manifests[manifest.digest]
+                manifest_to_list = models.ManifestListManifest(
+                    manifest_list=manifest,
+                    image_manifest=manifest_list,
+                    architecture=platform["architecture"],
+                    os=platform["os"],
+                    features=platform.get("features", ""),
+                    variant=platform.get("variant", ""),
+                    os_version=platform.get("os.version", ""),
+                    os_features=platform.get("os.features", ""),
+                )
+                manifests_to_list.append(manifest_to_list)
+
+            models.ManifestListManifest.objects.bulk_create(
+                manifests_to_list, ignore_conflicts=True, batch_size=1000
+            )
+            manifest = manifest_list
+        else:
+            # both docker/oci format should contain config, digest, mediaType, size
+            config_layer = content_data.get("config")
+            if not config_layer:
+                raise ManifestInvalid(
+                    digest=manifest_digest,
+                    reason="Pushing manifests of the version V1 is not supported",
+                )
+
+            config_digest = config_layer.get("digest")
+            try:
+                config_blob = models.Blob.objects.get(
+                    digest=config_digest, pk__in=repository.latest_version().content
+                )
+            except models.Blob.DoesNotExist:
+                raise BlobInvalid(digest=config_digest)
+
+            # both docker/oci format should contain layers, digest, media_type, size
+            layers = content_data.get("layers")
+            blobs = set()
+            for layer in layers:
+                layer_media_type = layer.get("mediaType")
+                urls = layer.get("urls")
+                if (
+                    layer_media_type
+                    in (
+                        models.MEDIA_TYPE.FOREIGN_BLOB,
+                        models.MEDIA_TYPE.FOREIGN_BLOB_OCI_TAR,
+                        models.MEDIA_TYPE.FOREIGN_BLOB_OCI_TAR_GZIP,
+                        models.MEDIA_TYPE.FOREIGN_BLOB_OCI_TAR_ZSTD,
+                    )
+                    and not urls
+                ):
+                    raise ManifestInvalid(
+                        digest=manifest_digest,
+                        reason="The URL of a foreign layer must be specified",
+                    )
+
+                digest = layer.get("digest")
+                blobs.add(digest)
+
+            blobs_qs = models.Blob.objects.filter(
+                digest__in=blobs, pk__in=repository.latest_version().content
+            )
+            if (len(blobs) - blobs_qs.count()) != 0:
+                raise ManifestInvalid(digest=manifest_digest)
+
+            manifest = self._save_manifest(artifact, manifest_digest, media_type, config_blob)
+
+            thru = []
+            for blob in blobs_qs:
+                thru.append(models.BlobManifest(manifest=manifest, manifest_blob=blob))
+            models.BlobManifest.objects.bulk_create(
+                objs=thru, ignore_conflicts=True, batch_size=1000
+            )
+
+        # a manifest cannot tagged by its digest - an identifier specified in the 'pk' parameter
+        if not pk.startswith("sha256:"):
+            tag = models.Tag(name=pk, tagged_manifest=manifest)
+            try:
+                tag.save()
+            except IntegrityError:
+                tag = models.Tag.objects.get(name=tag.name, tagged_manifest=manifest)
+                tag.touch()
+
+            tags_to_remove = models.Tag.objects.filter(
+                pk__in=repository.latest_version().content.all(), name=tag
+            ).exclude(tagged_manifest=manifest)
+            add_content_units = [str(tag.pk), str(manifest.pk)]
+            remove_content_units = [str(pk) for pk in tags_to_remove.values_list("pk")]
+        else:
+            add_content_units = [str(manifest.pk)]
+            remove_content_units = []
+
+        dispatched_task = dispatch(
+            add_and_remove,
+            exclusive_resources=[repository],
+            kwargs={
+                "repository_pk": str(repository.pk),
+                "add_content_units": add_content_units,
+                "remove_content_units": remove_content_units,
+            },
+        )
+
+        if has_task_completed(dispatched_task):
+            return ManifestResponse(manifest, path, request, status=201)
+
+    def _save_manifest(self, artifact, manifest_digest, content_type, config_blob=None):
         manifest = models.Manifest(
             digest=manifest_digest,
             schema_version=2,
-            media_type=request.content_type,
+            media_type=content_type,
             config_blob=config_blob,
         )
         try:
@@ -897,57 +1040,7 @@ class Manifests(RedirectsMixin, ContainerRegistryApiMixin, ViewSet):
             if not ca.artifact:
                 ca.artifact = artifact
                 ca.save(update_fields=["artifact"])
-        # both docker/oci format should contain layers, digest, media_type, size
-        layers = content_data.get("layers")
-        blobs = {}
-        for layer in layers:
-            blobs[layer.get("digest")] = layer.get("mediaType")
-        blobs_qs = models.Blob.objects.filter(digest__in=blobs.keys())
-        thru = []
-        for blob in blobs_qs:
-            # ensure there are no foreign layers
-            blob_media_type = blobs[blob.digest]
-            if blob_media_type not in (
-                models.MEDIA_TYPE.REGULAR_BLOB,
-                models.MEDIA_TYPE.REGULAR_BLOB_OCI,
-            ):
-                raise BlobInvalid(digest=blob.digest)
-            thru.append(models.BlobManifest(manifest=manifest, manifest_blob=blob))
-        models.BlobManifest.objects.bulk_create(objs=thru, ignore_conflicts=True, batch_size=1000)
-        tag = models.Tag(name=pk, tagged_manifest=manifest)
-        try:
-            tag.save()
-        except IntegrityError:
-            tag = models.Tag.objects.get(name=tag.name, tagged_manifest=manifest)
-            tag.touch()
-
-        tags_to_remove = models.Tag.objects.filter(
-            pk__in=repository.latest_version().content.all(), name=tag
-        ).exclude(tagged_manifest=manifest)
-        dispatched_task = dispatch(
-            add_and_remove,
-            exclusive_resources=[repository],
-            kwargs={
-                "repository_pk": str(repository.pk),
-                "add_content_units": [str(tag.pk), str(manifest.pk)],
-                "remove_content_units": [str(pk) for pk in tags_to_remove.values_list("pk")],
-            },
-        )
-
-        # Wait a small amount of time
-        for dummy in range(3):
-            time.sleep(1)
-            task = Task.objects.get(pk=dispatched_task.pk)
-            if task.state == "completed":
-                task.delete()
-                return ManifestResponse(manifest, path, request, status=201)
-            elif task.state in ["waiting", "running"]:
-                continue
-            else:
-                error = task.error
-                task.delete()
-                raise Exception(str(error))
-        raise Throttled()
+        return manifest
 
     def receive_artifact(self, chunk):
         """Handles assembling of Manifest as it's being uploaded."""
@@ -975,3 +1068,101 @@ class Manifests(RedirectsMixin, ContainerRegistryApiMixin, ViewSet):
                 artifact = Artifact.objects.get(sha256=artifact.sha256)
                 artifact.touch()
             return artifact
+
+
+class Signatures(ContainerRegistryApiMixin, ViewSet):
+    """A ViewSet for image signatures."""
+
+    lookup_value_regex = "sha256:[0-9a-f]{64}"
+
+    def head(self, request, path, pk=None):
+        """Respond to HEAD requests querying signatures by sha256."""
+        return self.get(request, path, pk=pk)
+
+    def get(self, request, path, pk):
+        """Return a signature identified by its sha256 checksum."""
+        _, _, repository_version = self.get_drv_pull(path)
+
+        try:
+            manifest = models.Manifest.objects.get(digest=pk, pk__in=repository_version.content)
+        except models.Manifest.DoesNotExist:
+            raise ManifestNotFound(reference=pk)
+
+        signatures = models.ManifestSignature.objects.filter(
+            signed_manifest=manifest, pk__in=repository_version.content
+        )
+
+        return Response(self.get_response_data(signatures))
+
+    @staticmethod
+    def get_response_data(signatures):
+        """Extract version, type, name, and content from the passed signature data."""
+        data = []
+        for signature in signatures:
+            signature = {
+                "schemaVersion": SIGNATURE_API_EXTENSION_VERSION,
+                "type": signature.type,
+                "name": signature.name,
+                "content": signature.data,
+            }
+            data.append(signature)
+        return {"signatures": data}
+
+    def put(self, request, path, pk):
+        """Create a new signature from the received data."""
+        _, repository = self.get_dr_push(request, path)
+
+        try:
+            manifest = models.Manifest.objects.get(
+                digest=pk, pk__in=repository.latest_version().content
+            )
+        except models.Manifest.DoesNotExist:
+            raise ManifestNotFound(reference=pk)
+
+        signature_payload = request.META["wsgi.input"].read(SIGNATURE_PAYLOAD_MAX_SIZE)
+        try:
+            signature_dict = json.loads(signature_payload)
+        except json.decoder.JSONDecodeError:
+            raise ManifestSignatureInvalid(digest=pk)
+
+        serializer = serializers.ManifestSignaturePutSerializer(data=signature_dict)
+        serializer.is_valid(raise_exception=True)
+
+        try:
+            signature_raw = base64.b64decode(signature_dict["content"])
+        except binascii.Error:
+            raise ManifestSignatureInvalid(digest=pk)
+
+        signature_json = extract_data_from_signature(signature_raw, manifest.digest)
+        if signature_json is None:
+            raise ManifestSignatureInvalid(digest=pk)
+
+        sig_digest = hashlib.sha256(signature_raw).hexdigest()
+        signature = models.ManifestSignature(
+            name=f"{manifest.digest}@{sig_digest[:32]}",
+            digest=f"sha256:{sig_digest}",
+            type=SIGNATURE_TYPE.ATOMIC_SHORT,
+            key_id=signature_json["signing_key_id"],
+            timestamp=signature_json["signature_timestamp"],
+            creator=signature_json["optional"].get("creator"),
+            data=signature_dict["content"],
+            signed_manifest=manifest,
+        )
+        try:
+            signature.save()
+        except IntegrityError:
+            signature = models.ManifestSignature.objects.get(digest=signature.digest)
+            signature.touch()
+
+        dispatched_task = dispatch(
+            add_and_remove,
+            exclusive_resources=[repository],
+            kwargs={
+                "repository_pk": str(repository.pk),
+                "add_content_units": [str(signature.pk)],
+                "remove_content_units": [],
+            },
+        )
+
+        if has_task_completed(dispatched_task):
+            return ManifestSignatureResponse(signature, path)
