@@ -2,19 +2,21 @@ from gettext import gettext as _
 import os
 import re
 
-from django.db import IntegrityError
-from django.core.exceptions import ObjectDoesNotExist
-
+from django.core.validators import URLValidator
 from rest_framework import serializers
 
 from pulpcore.plugin.models import (
     Artifact,
+    ContentRedirectContentGuard,
     Remote,
+    Repository,
     RepositoryVersion,
 )
 from pulpcore.plugin.serializers import (
-    ContentGuardSerializer,
+    ContentRedirectContentGuardSerializer,
     DetailRelatedField,
+    GetOrCreateSerializerMixin,
+    DistributionSerializer,
     IdentityField,
     ModelSerializer,
     NestedRelatedField,
@@ -22,36 +24,18 @@ from pulpcore.plugin.serializers import (
     RelatedField,
     RemoteSerializer,
     RepositorySerializer,
-    DistributionSerializer,
+    RepositorySyncURLSerializer,
     RepositoryVersionRelatedField,
     SingleArtifactContentSerializer,
-    validate_unknown_fields,
+    ValidateFieldsMixin,
 )
 
-from . import models
+from pulp_container.app import models
+from pulp_container.constants import SIGNATURE_TYPE
 
+VALID_SIGNATURE_NAME_REGEX = r"^sha256:[0-9a-f]{64}@[0-9a-f]{32}$"
 VALID_TAG_REGEX = r"^[A-Za-z0-9][A-Za-z0-9._-]*$"
-VALID_BASE_PATH_REGEX_COMPILED = re.compile(r"^[a-z0-9]+(?:(?:(?:[._]|__|[-]*)[a-z0-9]+)+)?$")
-
-
-class _GetOrCreateMixin:
-    @classmethod
-    def get_or_create(cls, natural_key, default_values=None):
-        try:
-            result = cls.Meta.model.objects.get(**natural_key)
-        except ObjectDoesNotExist:
-            data = {}
-            if default_values:
-                data.update(default_values)
-            data.update(natural_key)
-            serializer = cls(data=data)
-            try:
-                serializer.is_valid(raise_exception=True)
-                result = serializer.create(serializer.validated_data)
-            except (IntegrityError, serializers.ValidationError):
-                # recover from a race condition, where another thread just created the object
-                result = cls.Meta.model.objects.get(**natural_key)
-        return result
+VALID_BASE_PATH_REGEX_COMPILED = re.compile(r"^[a-z0-9]+(?:(?:[._]|__|[-]*)[a-z0-9])*$")
 
 
 class TagSerializer(NoArtifactContentSerializer):
@@ -118,13 +102,56 @@ class BlobSerializer(SingleArtifactContentSerializer):
     """
 
     digest = serializers.CharField(help_text="sha256 of the Blob file")
-    media_type = serializers.CharField(help_text="Blob media type of the file")
 
     class Meta:
-        fields = SingleArtifactContentSerializer.Meta.fields + ("digest", "media_type")
+        fields = SingleArtifactContentSerializer.Meta.fields + ("digest",)
         model = models.Blob
 
 
+class ManifestSignatureSerializer(NoArtifactContentSerializer):
+    """
+    Serializer for image manifest signatures.
+    """
+
+    name = serializers.CharField(
+        help_text="Signature name in the format of `digest_algo:manifest_digest@random_32_chars`"
+    )
+    digest = serializers.CharField(help_text="sha256 digest of the signature blob")
+    type = serializers.CharField(help_text="Container signature type, e.g. 'atomic'")
+    key_id = serializers.CharField(help_text="Signing key ID")
+    timestamp = serializers.IntegerField(help_text="Timestamp of a signature")
+    creator = serializers.CharField(help_text="Signature creator")
+    signed_manifest = DetailRelatedField(
+        many=False,
+        help_text="Manifest that is signed",
+        view_name="container-manifests-detail",
+        queryset=models.Manifest.objects.all(),
+    )
+
+    class Meta:
+        fields = NoArtifactContentSerializer.Meta.fields + (
+            "name",
+            "digest",
+            "type",
+            "key_id",
+            "timestamp",
+            "creator",
+            "signed_manifest",
+        )
+        model = models.ManifestSignature
+
+
+class ManifestSignaturePutSerializer(serializers.Serializer):
+    """
+    A serializer for image signatures provided in a PUT request.
+    """
+
+    name = serializers.RegexField(regex=VALID_SIGNATURE_NAME_REGEX)
+    schemaVersion = serializers.IntegerField(max_value=2, min_value=2)
+    type = serializers.ChoiceField([SIGNATURE_TYPE.ATOMIC_SHORT])
+    content = serializers.CharField()
+
+
 class RegistryPathField(serializers.CharField):
     """
     Serializer Field for the registry_path field of the ContainerDistribution.
@@ -138,7 +165,7 @@ class RegistryPathField(serializers.CharField):
         return f"{request.get_host()}/{value}"
 
 
-class ContainerNamespaceSerializer(ModelSerializer, _GetOrCreateMixin):
+class ContainerNamespaceSerializer(ModelSerializer, GetOrCreateSerializerMixin):
     """
     Serializer for ContainerNamespaces.
     """
@@ -155,8 +182,17 @@ class ContainerRepositorySerializer(RepositorySerializer):
     Serializer for Container Repositories.
     """
 
+    manifest_signing_service = RelatedField(
+        help_text="A reference to an associated signing service.",
+        view_name="signing-services-detail",
+        queryset=models.ManifestSigningService.objects.all(),
+        many=False,
+        required=False,
+        allow_null=True,
+    )
+
     class Meta:
-        fields = RepositorySerializer.Meta.fields
+        fields = RepositorySerializer.Meta.fields + ("manifest_signing_service",)
         model = models.ContainerRepository
 
 
@@ -165,8 +201,19 @@ class ContainerPushRepositorySerializer(RepositorySerializer):
     Serializer for Container Push Repositories.
     """
 
+    manifest_signing_service = RelatedField(
+        help_text="A reference to an associated signing service.",
+        view_name="signing-services-detail",
+        queryset=models.ManifestSigningService.objects.all(),
+        many=False,
+        required=False,
+        allow_null=True,
+    )
+
     class Meta:
-        fields = tuple(set(RepositorySerializer.Meta.fields) - set(["remote"]))
+        fields = tuple(
+            set(RepositorySerializer.Meta.fields + ("manifest_signing_service",)) - set(["remote"])
+        )
         model = models.ContainerPushRepository
 
 
@@ -214,8 +261,19 @@ class ContainerRemoteSerializer(RemoteSerializer):
         default=Remote.IMMEDIATE,
     )
 
+    sigstore = serializers.CharField(
+        required=False,
+        help_text=_("A URL to a sigstore to download image signatures from"),
+        validators=[URLValidator(schemes=["http", "https"])],
+    )
+
     class Meta:
-        fields = RemoteSerializer.Meta.fields + ("upstream_name", "include_tags", "exclude_tags")
+        fields = RemoteSerializer.Meta.fields + (
+            "upstream_name",
+            "include_tags",
+            "exclude_tags",
+            "sigstore",
+        )
         model = models.ContainerRemote
 
 
@@ -228,7 +286,7 @@ class ContainerDistributionSerializer(DistributionSerializer):
         source="base_path",
         read_only=True,
         help_text=_(
-            "The Registry hostame/name/ to use with docker pull command defined by "
+            "The Registry hostname/name/ to use with docker pull command defined by "
             "this distribution."
         ),
     )
@@ -236,7 +294,7 @@ class ContainerDistributionSerializer(DistributionSerializer):
         required=False,
         help_text=_("An optional content-guard. If none is specified, a default one will be used."),
         view_name=r"contentguards-container/content-redirect-detail",
-        queryset=models.ContentRedirectContentGuard.objects.all(),
+        queryset=ContentRedirectContentGuard.objects.all(),
         allow_null=False,
     )
     namespace = RelatedField(
@@ -275,19 +333,12 @@ class ContainerDistributionSerializer(DistributionSerializer):
                     )
                 )
 
-        if "base_path" in validated_data:
-            base_path = validated_data["base_path"]
+        base_path = validated_data.get("base_path")
+        if base_path:
             namespace_name = base_path.split("/")[0]
-
-            if validated_data.get("namespace"):
-                if validated_data["namespace"].name != namespace_name:
-                    raise serializers.ValidationError(
-                        _("Selected namespace does not match first component of base_path.")
-                    )
-            else:
-                validated_data["namespace"] = ContainerNamespaceSerializer.get_or_create(
-                    {"name": namespace_name}
-                )
+            validated_data["namespace"] = ContainerNamespaceSerializer.get_or_create(
+                {"name": namespace_name}
+            )
         return validated_data
 
     def validate_base_path(self, value):
@@ -315,16 +366,7 @@ class ContainerDistributionSerializer(DistributionSerializer):
         )
 
 
-class ContentRedirectContentGuardSerializer(ContentGuardSerializer, _GetOrCreateMixin):
-    """
-    A serializer for ContentRedirectContentGuard.
-    """
-
-    class Meta(ContentGuardSerializer.Meta):
-        model = models.ContentRedirectContentGuard
-
-
-class TagOperationSerializer(serializers.Serializer):
+class TagOperationSerializer(ValidateFieldsMixin, serializers.Serializer):
     """
     A base serializer for tagging and untagging manifests.
     """
@@ -342,30 +384,6 @@ class TagOperationSerializer(serializers.Serializer):
         },
     )
 
-    def validate(self, data):
-        """
-        Validate data passed through a request call.
-
-        Check if a repository has got a reference to a latest repository version. A
-        new dictionary object is initialized by the passed data and altered by a latest
-        repository version.
-        """
-        new_data = {}
-        new_data.update(self.initial_data)
-
-        latest_version = new_data["repository"].latest_version()
-        if not latest_version:
-            raise serializers.ValidationError(
-                _(
-                    "The latest repository version of '{}' was not found".format(
-                        new_data["repository"]
-                    )
-                )
-            )
-
-        new_data["latest_version"] = latest_version
-        return new_data
-
 
 class TagImageSerializer(TagOperationSerializer):
     """
@@ -383,10 +401,11 @@ class TagImageSerializer(TagOperationSerializer):
         method checks if the tag exists within the repository.
         """
         new_data = super().validate(data)
+        latest_version = self.context["repository"].latest_version()
 
         try:
             manifest = models.Manifest.objects.get(
-                pk__in=new_data["latest_version"].content.all(), digest=new_data["digest"]
+                pk__in=latest_version.content.all(), digest=new_data["digest"]
             )
             manifest.touch()
         except models.Manifest.DoesNotExist:
@@ -394,7 +413,7 @@ class TagImageSerializer(TagOperationSerializer):
                 _(
                     "A manifest with the digest '{}' does not "
                     "exist in the latest repository version '{}'".format(
-                        new_data["digest"], new_data["latest_version"]
+                        new_data["digest"], latest_version
                     )
                 )
             )
@@ -415,16 +434,15 @@ class UnTagImageSerializer(TagOperationSerializer):
         The method checks if the tag exists within the latest repository version.
         """
         new_data = super().validate(data)
+        latest_version = self.context["repository"].latest_version()
 
         try:
-            models.Tag.objects.get(
-                pk__in=new_data["latest_version"].content.all(), name=new_data["tag"]
-            )
+            models.Tag.objects.get(pk__in=latest_version.content.all(), name=new_data["tag"])
         except models.Tag.DoesNotExist:
             raise serializers.ValidationError(
                 _(
                     "The tag '{}' does not exist in the latest repository version '{}'".format(
-                        new_data["tag"], new_data["latest_version"]
+                        new_data["tag"], latest_version
                     )
                 )
             )
@@ -432,7 +450,7 @@ class UnTagImageSerializer(TagOperationSerializer):
         return new_data
 
 
-class RecursiveManageSerializer(serializers.Serializer):
+class RecursiveManageSerializer(ValidateFieldsMixin, serializers.Serializer):
     """
     Serializer for adding and removing content to/from a Container repository.
     """
@@ -445,6 +463,8 @@ class RecursiveManageSerializer(serializers.Serializer):
         """
         Validate data passed through a request call.
         """
+        data = super().validate(data)
+
         content_units = data.get("content_units", None)
         if content_units:
             if "*" in content_units and len(content_units) > 1:
@@ -454,7 +474,7 @@ class RecursiveManageSerializer(serializers.Serializer):
         return data
 
 
-class CopySerializer(serializers.Serializer):
+class CopySerializer(ValidateFieldsMixin, serializers.Serializer):
     """
     Serializer for copying units from a source repository to a destination repository.
     """
@@ -476,28 +496,21 @@ class CopySerializer(serializers.Serializer):
     )
 
     def validate(self, data):
-        """Ensure that source_repository or source_rpository_version is pass, but not both."""
-        if hasattr(self, "initial_data"):
-            validate_unknown_fields(self.initial_data, self.fields)
+        """Ensure that source_repository or source_repository_version is passed, but not both."""
+        data = super().validate(data)
 
         repository = data.pop("source_repository", None)
         repository_version = data.get("source_repository_version")
         if not repository and not repository_version:
             raise serializers.ValidationError(
-                _("Either the 'repository' or 'repository_version' need to be specified")
+                _("Either the 'repository' or 'repository_version' needs to be specified")
             )
         elif not repository and repository_version:
             return data
         elif repository and not repository_version:
-            version = repository.latest_version()
-            if version:
-                new_data = {"source_repository_version": version}
-                new_data.update(data)
-                return new_data
-            else:
-                raise serializers.ValidationError(
-                    detail=_("Source repository has no version available to copy content from")
-                )
+            new_data = {"source_repository_version": repository.latest_version()}
+            new_data.update(data)
+            return new_data
         raise serializers.ValidationError(
             _(
                 "Either the 'repository' or 'repository_version' need to be specified "
@@ -531,7 +544,7 @@ class ManifestCopySerializer(CopySerializer):
     )
 
 
-class RemoveImageSerializer(serializers.Serializer):
+class RemoveImageSerializer(ValidateFieldsMixin, serializers.Serializer):
     """
     A serializer for parsing and validating data associated with the image removal.
     """
@@ -540,33 +553,22 @@ class RemoveImageSerializer(serializers.Serializer):
 
     def validate(self, data):
         """
-        Validate and extract the latest repository version, manifest, and tags from the passed data.
+        Validate and extract the latest repository version, manifest, signatures and tags
+        from the passed data.
         """
-        new_data = {}
-        new_data.update(self.initial_data)
-
-        latest_version = new_data["repository"].latest_version()
-        if not latest_version:
-            raise serializers.ValidationError(
-                _(
-                    "The latest repository version of '{}' was not found".format(
-                        new_data["repository"]
-                    )
-                )
-            )
-
-        new_data["latest_version"] = latest_version
+        new_data = super().validate(data)
+        latest_version = self.context["repository"].latest_version()
 
         try:
             manifest = models.Manifest.objects.get(
-                pk__in=new_data["latest_version"].content.all(), digest=new_data["digest"]
+                pk__in=latest_version.content.all(), digest=new_data["digest"]
             )
         except models.Manifest.DoesNotExist:
             raise serializers.ValidationError(
                 _(
                     "A manifest with the digest '{}' does not "
                     "exist in the latest repository version '{}'".format(
-                        new_data["digest"], new_data["latest_version"]
+                        new_data["digest"], latest_version
                     )
                 )
             )
@@ -574,14 +576,52 @@ class RemoveImageSerializer(serializers.Serializer):
         new_data["manifest"] = manifest
 
         tags_pks = models.Tag.objects.filter(
-            pk__in=new_data["latest_version"].content.all(), tagged_manifest=new_data["manifest"]
+            pk__in=latest_version.content.all(), tagged_manifest=new_data["manifest"]
         ).values_list("pk", flat=True)
         new_data["tags_pks"] = tags_pks
+        sigs_pks = models.ManifestSignature.objects.filter(
+            pk__in=latest_version.content.all(), signed_manifest=new_data["manifest"]
+        ).values_list("pk", flat=True)
+        new_data["sigs_pks"] = sigs_pks
 
         return new_data
 
 
-class OCIBuildImageSerializer(serializers.Serializer):
+class RemoveSignaturesSerializer(ValidateFieldsMixin, serializers.Serializer):
+    """
+    A serializer for parsing and validating data associated with the signatures removal.
+    """
+
+    signed_with_key_id = serializers.CharField(
+        help_text="key_id of the key the signatures were produced with"
+    )
+
+    def validate(self, data):
+        """
+        Validate and extract the latest repository version, signatures from the passed data.
+        """
+        new_data = super().validate(data)
+        latest_version = self.context["repository"].latest_version()
+
+        sigs_pks = models.ManifestSignature.objects.filter(
+            pk__in=latest_version.content.all(), key_id=new_data["signed_with_key_id"]
+        ).values_list("pk", flat=True)
+        if not sigs_pks:
+            raise serializers.ValidationError(
+                _(
+                    "There are no signatures in the latest repository version '{}' "
+                    "produced with the specified key_id '{}'".format(
+                        latest_version, new_data["signed_with_key_id"]
+                    )
+                )
+            )
+
+        new_data["sigs_pks"] = sigs_pks
+
+        return new_data
+
+
+class OCIBuildImageSerializer(ValidateFieldsMixin, serializers.Serializer):
     """
     Serializer for building an OCI container image from a Containerfile.
 
@@ -594,10 +634,12 @@ class OCIBuildImageSerializer(serializers.Serializer):
         lookup_field="pk",
         view_name="artifacts-detail",
         queryset=Artifact.objects.all(),
-        help_text=_("Artifact representing the Containerfile that should be used to run buildah."),
+        help_text=_(
+            "Artifact representing the Containerfile that should be used to run podman-build."
+        ),
     )
     containerfile = serializers.FileField(
-        help_text=_("An uploaded Containerfile that should be used to run buildah."),
+        help_text=_("An uploaded Containerfile that should be used to run podman-build."),
         required=False,
     )
     tag = serializers.CharField(
@@ -647,7 +689,7 @@ class OCIBuildImageSerializer(serializers.Serializer):
                 try:
                     artifact = artifactfield.run_validation(data=url)
                     artifact.touch()
-                    artifacts[artifact.pk] = relative_path
+                    artifacts[str(artifact.pk)] = relative_path
                 except serializers.ValidationError as e:
                     # Append the URL of missing Artifact to the error message
                     e.detail[0] = "%s %s" % (e.detail[0], url)
@@ -663,3 +705,84 @@ class OCIBuildImageSerializer(serializers.Serializer):
             "tag",
             "artifacts",
         )
+
+
+class ContainerRepositorySyncURLSerializer(RepositorySyncURLSerializer):
+    """
+    Serializer for Container Sync.
+    """
+
+    signed_only = serializers.BooleanField(
+        required=False,
+        default=False,
+        help_text=_(
+            "If ``True``, only signed content will be synced. Signatures are not verified."
+        ),
+    )
+
+
+class RepositorySignSerializer(ValidateFieldsMixin, serializers.Serializer):
+    """
+    Serializer for container images signing.
+    """
+
+    manifest_signing_service = RelatedField(
+        required=False,
+        many=False,
+        view_name="signing-services-detail",
+        queryset=models.ManifestSigningService.objects.all(),
+        help_text=_(
+            "A signing service to sign with. This will override a signing service set on the repo."
+        ),
+        allow_null=True,
+    )
+
+    # Ask for the future_base_path for synced repos - this should match future/existing distribution
+    # otherwise client verification can fail, it looks at 'docker-reference' in the signature json
+    future_base_path = serializers.CharField(
+        required=False,
+        help_text=_("Future base path content will be distributed at for sync repos"),
+    )
+    tags_list = serializers.ListField(help_text=_("A list of tags to sign."), required=False)
+
+    def validate(self, data):
+        """Ensure that future_base_path is provided for synced repos."""
+
+        data = super().validate(data)
+
+        repository = Repository.objects.get(pk=self.context["repository_pk"]).cast()
+        try:
+            signing_service = repository.manifest_signing_service
+        except KeyError:
+            signing_service = None
+
+        if "manifest_signing_service" not in data and not signing_service:
+            raise serializers.ValidationError(
+                {
+                    "manifest_signing_service": _(
+                        "This field is required since a signing_service is not set on the repo."
+                    )
+                }
+            )
+
+        if repository.PUSH_ENABLED:
+            if "future_base_path" in data:
+                raise serializers.ValidationError(
+                    {
+                        "future_base_path": _(
+                            "This field cannot be set since this is a push repo type."
+                        )
+                    }
+                )
+            data["future_base_path"] = repository.distributions.first().base_path
+        else:
+            if "future_base_path" not in data:
+                raise serializers.ValidationError(
+                    {
+                        "future_base_path": _(
+                            "This field is required since this is a sync repo type."
+                        )
+                    }
+                )
+
+        return data
