@@ -13,7 +13,7 @@ from pulp_container.app.models import (
     Tag,
 )
 from pulp_container.constants import MEDIA_TYPE
-from pulpcore.plugin.models import Artifact, ContentArtifact
+from pulpcore.plugin.models import Artifact, ContentArtifact, Content
 
 
 def get_or_create_blob(layer_json, manifest, path):
@@ -33,15 +33,15 @@ def get_or_create_blob(layer_json, manifest, path):
         blob = Blob.objects.get(digest=layer_json["digest"])
         blob.touch()
     except Blob.DoesNotExist:
-        layer_file_name = "{}{}".format(path, layer_json["digest"][7:])
+        layer_file_name = os.path.join(path, layer_json["digest"][7:])
         layer_artifact = Artifact.init_and_validate(layer_file_name)
         layer_artifact.save()
-        blob = Blob(digest=layer_json["digest"], media_type=layer_json["mediaType"])
+        blob = Blob(digest=layer_json["digest"])
         blob.save()
         ContentArtifact(
             artifact=layer_artifact, content=blob, relative_path=layer_json["digest"]
         ).save()
-    if blob.media_type != MEDIA_TYPE.CONFIG_BLOB_OCI:
+    if layer_json["mediaType"] != MEDIA_TYPE.CONFIG_BLOB_OCI:
         BlobManifest(manifest=manifest, manifest_blob=blob).save()
     return blob
 
@@ -60,7 +60,7 @@ def add_image_from_directory_to_repository(path, repository, tag):
         image and tag.
 
     """
-    manifest_path = "{}manifest.json".format(path)
+    manifest_path = os.path.join(path, "manifest.json")
     manifest_artifact = Artifact.init_and_validate(manifest_path)
     manifest_artifact.save()
     manifest_digest = "sha256:{}".format(manifest_artifact.sha256)
@@ -73,23 +73,27 @@ def add_image_from_directory_to_repository(path, repository, tag):
     ).save()
     tag = Tag(name=tag, tagged_manifest=manifest)
     tag.save()
+
     with repository.new_version() as new_repo_version:
-        new_repo_version.add_content(Manifest.objects.filter(pk=manifest.pk))
-        new_repo_version.add_content(Tag.objects.filter(pk=tag.pk))
-        with open(manifest_artifact.file.path, "r") as manifest_file:
-            manifest_json = json.load(manifest_file)
-            config_blob = get_or_create_blob(manifest_json["config"], manifest, path)
-            manifest.config_blob = config_blob
-            manifest.save()
-            new_repo_version.add_content(Blob.objects.filter(pk=config_blob.pk))
-            for layer in manifest_json["layers"]:
-                blob = get_or_create_blob(layer, manifest, path)
-                new_repo_version.add_content(Blob.objects.filter(pk=blob.pk))
+        manifest_json = json.load(manifest_artifact.file)
+        manifest_artifact.file.close()
+
+        config_blob = get_or_create_blob(manifest_json["config"], manifest, path)
+        manifest.config_blob = config_blob
+        manifest.save()
+
+        pks_to_add = []
+        for layer in manifest_json["layers"]:
+            pks_to_add.append(get_or_create_blob(layer, manifest, path).pk)
+
+        pks_to_add.extend([manifest.pk, tag.pk, config_blob.pk])
+        new_repo_version.add_content(Content.objects.filter(pk__in=pks_to_add))
+
     return new_repo_version
 
 
 def build_image_from_containerfile(
-    containerfile_pk=None, artifacts={}, repository_pk=None, tag=None
+    containerfile_pk=None, artifacts=None, repository_pk=None, tag=None
 ):
     """
     Builds an OCI container image from a Containerfile.
@@ -113,26 +117,44 @@ def build_image_from_containerfile(
     containerfile = Artifact.objects.get(pk=containerfile_pk)
     repository = ContainerRepository.objects.get(pk=repository_pk)
     name = str(uuid4())
-    with tempfile.TemporaryDirectory(".") as working_directory:
-        path = "{}/".format(working_directory.path)
+    with tempfile.TemporaryDirectory(dir=".") as working_directory:
+        working_directory = os.path.abspath(working_directory)
+        context_path = os.path.join(working_directory, "context")
+        os.makedirs(context_path)
         for key, val in artifacts.items():
             artifact = Artifact.objects.get(pk=key)
-            dirs = os.path.split(val)[0]
+            dest_path = os.path.join(context_path, val)
+            dirs = os.path.split(dest_path)[0]
             if dirs:
                 os.makedirs(dirs)
+            with open(dest_path, "wb") as dest:
+                shutil.copyfileobj(artifact.file, dest)
 
-            shutil.copy(artifact.file.path, "{}{}".format(path, val))
+            containerfile_path = os.path.join(working_directory, "Containerfile")
+
+        with open(containerfile_path, "wb") as dest:
+            shutil.copyfileobj(containerfile.file, dest)
         bud_cp = subprocess.run(
-            ["buildah", "bud", "-f", containerfile.file.path, "-t", name],
+            [
+                "podman",
+                "build",
+                "-f",
+                containerfile_path,
+                "-t",
+                name,
+                context_path,
+                "--isolation",
+                "rootless",
+            ],
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE,
         )
         if bud_cp.returncode != 0:
             raise Exception(bud_cp.stderr)
-        image_dir = "{}image/".format(path)
+        image_dir = os.path.join(working_directory, "image")
         os.makedirs(image_dir)
         push_cp = subprocess.run(
-            ["buildah", "push", "-f", "oci", name, "dir:{}".format(image_dir)],
+            ["podman", "push", "-f", "oci", name, "dir:{}".format(image_dir)],
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE,
         )
