@@ -1,10 +1,8 @@
-from gettext import gettext as _
 import logging
 
 from pulpcore.plugin.stages import (
     ArtifactDownloader,
     ArtifactSaver,
-    ContentSaver,
     DeclarativeVersion,
     RemoteArtifactSaver,
     ResolveContentFutures,
@@ -12,14 +10,14 @@ from pulpcore.plugin.stages import (
     QueryExistingContents,
 )
 
-from .sync_stages import InterrelateContent, ContainerFirstStage
+from .sync_stages import ContainerFirstStage, ContainerContentSaver
 from pulp_container.app.models import ContainerRemote, ContainerRepository
 
 
 log = logging.getLogger(__name__)
 
 
-def synchronize(remote_pk, repository_pk, mirror):
+def synchronize(remote_pk, repository_pk, mirror, signed_only):
     """
     Sync content from the remote repository.
 
@@ -29,6 +27,7 @@ def synchronize(remote_pk, repository_pk, mirror):
         remote_pk (str): The remote PK.
         repository_pk (str): The repository PK.
         mirror (boolean): A boolean indicating enabled or disabled mirror mode.
+        signed_only (boolean): A boolean indicating whether to sync only signed content or all.
 
     Raises:
         ValueError: If the remote does not specify a URL to sync
@@ -36,10 +35,8 @@ def synchronize(remote_pk, repository_pk, mirror):
     """
     remote = ContainerRemote.objects.get(pk=remote_pk)
     repository = ContainerRepository.objects.get(pk=repository_pk)
-    if not remote.url:
-        raise ValueError(_("A remote must have a url specified to synchronize."))
-    log.info(_("Synchronizing: repository={r} remote={p}").format(r=repository.name, p=remote.name))
-    first_stage = ContainerFirstStage(remote)
+    log.info("Synchronizing: repository={r} remote={p}".format(r=repository.name, p=remote.name))
+    first_stage = ContainerFirstStage(remote, signed_only)
     dv = ContainerDeclarativeVersion(first_stage, repository, mirror)
     return dv.create()
 
@@ -69,10 +66,9 @@ class ContainerDeclarativeVersion(DeclarativeVersion):
             ArtifactDownloader(),
             ArtifactSaver(),
             QueryExistingContents(),
-            ContentSaver(),
+            ContainerContentSaver(),
             RemoteArtifactSaver(),
             ResolveContentFutures(),
-            InterrelateContent(),
         ]
 
         return pipeline
