@@ -1,10 +1,10 @@
-# coding=utf-8
 """Tests that verify that images served by Pulp can be pulled."""
 import contextlib
 import hashlib
 import json
 import requests
 import unittest
+
 from urllib.parse import urljoin, urlparse
 
 from pulp_smash import api, cli, config, exceptions
@@ -16,26 +16,24 @@ from pulp_smash.pulp3.utils import (
 )
 
 from pulp_container.tests.functional.utils import (
-    TOKEN_AUTH_DISABLED,
     core_client,
     gen_container_client,
     gen_container_remote,
-    get_docker_hub_remote_blobsums,
-    BearerTokenAuth,
-    AuthenticationHeaderQueries,
+    get_blobsums_from_remote_registry,
+    get_auth_for_url,
 )
 from pulp_container.tests.functional.constants import (
     CONTAINER_CONTENT_NAME,
-    REPO_UPSTREAM_NAME,
-    REPO_UPSTREAM_TAG,
+    REGISTRY_V2_REPO_HELLO_WORLD,
+    PULP_HELLO_WORLD_LINUX_TAG,
 )
 from pulp_container.constants import EMPTY_BLOB, MEDIA_TYPE
 
 from pulpcore.client.pulp_container import (
     ContainerContainerDistribution,
     ContainerContainerRepository,
+    ContainerRepositorySyncURL,
     DistributionsContainerApi,
-    RepositorySyncURL,
     RepositoriesContainerApi,
     RemotesContainerApi,
 )
@@ -85,7 +83,7 @@ class PullContentTestCase(unittest.TestCase):
             cls.teardown_cleanups.append((cls.remotes_api.delete, cls.remote.pulp_href))
 
             # Step 3
-            sync_data = RepositorySyncURL(remote=cls.remote.pulp_href)
+            sync_data = ContainerRepositorySyncURL(remote=cls.remote.pulp_href)
             sync_response = cls.repositories_api.sync(_repo.pulp_href, sync_data)
             monitor_task(sync_response.task)
             cls.repo = cls.repositories_api.read(_repo.pulp_href)
@@ -139,7 +137,7 @@ class PullContentTestCase(unittest.TestCase):
         # Assert that at least one layer is synced from remote:latest
         # and the checksum matched with remote
         self.assertTrue(
-            any([result["blobSum"] in checksums for result in get_docker_hub_remote_blobsums()]),
+            any([checksum in checksums for checksum in get_blobsums_from_remote_registry()]),
             "Cannot find a matching layer on remote registry.",
         )
 
@@ -148,27 +146,9 @@ class PullContentTestCase(unittest.TestCase):
         image_path = "/v2/{}/manifests/{}".format(self.distribution_with_repo.base_path, "latest")
         latest_image_url = urljoin(self.cfg.get_base_url(), image_path)
 
-        if TOKEN_AUTH_DISABLED:
-            auth = ()
-        else:
-            with self.assertRaises(requests.HTTPError) as cm:
-                self.client.get(latest_image_url, headers={"Accept": MEDIA_TYPE.MANIFEST_V1})
-
-            content_response = cm.exception.response
-            self.assertEqual(content_response.status_code, 401)
-
-            authenticate_header = content_response.headers["Www-Authenticate"]
-            queries = AuthenticationHeaderQueries(authenticate_header)
-            content_response = requests.get(
-                queries.realm, params={"service": queries.service, "scope": queries.scope}
-            )
-            content_response.raise_for_status()
-            token = content_response.json()["token"]
-            auth = BearerTokenAuth(token)
+        auth = get_auth_for_url(latest_image_url)
         content_response = requests.get(
-            latest_image_url,
-            auth=auth,
-            headers={"Accept": MEDIA_TYPE.MANIFEST_V1},
+            latest_image_url, auth=auth, headers={"Accept": MEDIA_TYPE.MANIFEST_V1}
         )
         content_response.raise_for_status()
         base_content_type = content_response.headers["Content-Type"].split(";")[0]
@@ -194,22 +174,7 @@ class PullContentTestCase(unittest.TestCase):
         blob_path = "/v2/{}/blobs/{}".format(self.distribution_with_repo.base_path, EMPTY_BLOB)
         empty_blob_url = urljoin(self.cfg.get_base_url(), blob_path)
 
-        if TOKEN_AUTH_DISABLED:
-            auth = ()
-        else:
-            with self.assertRaises(requests.HTTPError) as cm:
-                requests.get(empty_blob_url).raise_for_status()
-
-            content_response = cm.exception.response
-            self.assertEqual(content_response.status_code, 401)
-
-            authenticate_header = content_response.headers["Www-Authenticate"]
-            queries = AuthenticationHeaderQueries(authenticate_header)
-            content_response = requests.get(
-                queries.realm, params={"service": queries.service, "scope": queries.scope}
-            )
-            content_response.raise_for_status()
-            auth = BearerTokenAuth(content_response.json()["token"])
+        auth = get_auth_for_url(empty_blob_url)
         content_response = requests.get(empty_blob_url, auth=auth)
         content_response.raise_for_status()
         # calculate digest of the payload
@@ -236,11 +201,11 @@ class PullContentTestCase(unittest.TestCase):
         self.teardown_cleanups.append((registry.rmi, local_url))
         local_image = registry.inspect(local_url)
 
-        registry.pull(REPO_UPSTREAM_NAME)
-        remote_image = registry.inspect(REPO_UPSTREAM_NAME)
+        registry.pull(REGISTRY_V2_REPO_HELLO_WORLD)
+        remote_image = registry.inspect(REGISTRY_V2_REPO_HELLO_WORLD)
 
         self.assertEqual(local_image[0]["Id"], remote_image[0]["Id"])
-        registry.rmi(REPO_UPSTREAM_NAME)
+        registry.rmi(REGISTRY_V2_REPO_HELLO_WORLD)
 
     def test_pull_image_from_repository_version(self):
         """Verify that a client can pull the image from Pulp.
@@ -260,11 +225,11 @@ class PullContentTestCase(unittest.TestCase):
         self.teardown_cleanups.append((registry.rmi, local_url))
         local_image = registry.inspect(local_url)
 
-        registry.pull(REPO_UPSTREAM_NAME)
-        remote_image = registry.inspect(REPO_UPSTREAM_NAME)
+        registry.pull(REGISTRY_V2_REPO_HELLO_WORLD)
+        remote_image = registry.inspect(REGISTRY_V2_REPO_HELLO_WORLD)
 
         self.assertEqual(local_image[0]["Id"], remote_image[0]["Id"])
-        registry.rmi(REPO_UPSTREAM_NAME)
+        registry.rmi(REGISTRY_V2_REPO_HELLO_WORLD)
 
     def test_pull_image_with_tag(self):
         """Verify that a client can pull the image from Pulp with a tag.
@@ -280,16 +245,18 @@ class PullContentTestCase(unittest.TestCase):
 
         local_url = (
             urljoin(self.cfg.get_base_url(), self.distribution_with_repo.base_path)
-            + REPO_UPSTREAM_TAG
+            + PULP_HELLO_WORLD_LINUX_TAG
         )
 
         registry.pull(local_url)
         self.teardown_cleanups.append((registry.rmi, local_url))
         local_image = registry.inspect(local_url)
 
-        registry.pull(REPO_UPSTREAM_NAME + REPO_UPSTREAM_TAG)
-        self.teardown_cleanups.append((registry.rmi, REPO_UPSTREAM_NAME + REPO_UPSTREAM_TAG))
-        remote_image = registry.inspect(REPO_UPSTREAM_NAME + REPO_UPSTREAM_TAG)
+        registry.pull(REGISTRY_V2_REPO_HELLO_WORLD + PULP_HELLO_WORLD_LINUX_TAG)
+        self.teardown_cleanups.append(
+            (registry.rmi, REGISTRY_V2_REPO_HELLO_WORLD + PULP_HELLO_WORLD_LINUX_TAG)
+        )
+        remote_image = registry.inspect(REGISTRY_V2_REPO_HELLO_WORLD + PULP_HELLO_WORLD_LINUX_TAG)
 
         self.assertEqual(local_image[0]["Id"], remote_image[0]["Id"])
 
@@ -350,7 +317,7 @@ class PullOnDemandContentTestCase(unittest.TestCase):
             cls.teardown_cleanups.append((cls.remotes_api.delete, cls.remote.pulp_href))
 
             # Step 3
-            sync_data = RepositorySyncURL(remote=cls.remote.pulp_href)
+            sync_data = ContainerRepositorySyncURL(remote=cls.remote.pulp_href)
             sync_response = cls.repositories_api.sync(_repo.pulp_href, sync_data)
             monitor_task(sync_response.task)
 
@@ -408,7 +375,7 @@ class PullOnDemandContentTestCase(unittest.TestCase):
         # Assert that at least one layer is synced from remote:latest
         # and the checksum matched with remote
         self.assertTrue(
-            any([result["blobSum"] in checksums for result in get_docker_hub_remote_blobsums()]),
+            any([checksum in checksums for checksum in get_blobsums_from_remote_registry()]),
             "Cannot find a matching layer on remote registry.",
         )
 
@@ -431,15 +398,15 @@ class PullOnDemandContentTestCase(unittest.TestCase):
         self.teardown_cleanups.append((registry.rmi, local_url))
         local_image = registry.inspect(local_url)
 
-        registry.pull(REPO_UPSTREAM_NAME)
-        remote_image = registry.inspect(REPO_UPSTREAM_NAME)
+        registry.pull(REGISTRY_V2_REPO_HELLO_WORLD)
+        remote_image = registry.inspect(REGISTRY_V2_REPO_HELLO_WORLD)
 
         self.assertEqual(local_image[0]["Id"], remote_image[0]["Id"])
 
         new_artifact_count = self.artifacts_api.list().count
         self.assertGreater(new_artifact_count, self.artifact_count)
 
-        registry.rmi(REPO_UPSTREAM_NAME)
+        registry.rmi(REGISTRY_V2_REPO_HELLO_WORLD)
 
     def test_pull_image_from_repository_version(self):
         """Verify that a client can pull the image from Pulp (on-demand).
@@ -459,11 +426,11 @@ class PullOnDemandContentTestCase(unittest.TestCase):
         self.teardown_cleanups.append((registry.rmi, local_url))
         local_image = registry.inspect(local_url)
 
-        registry.pull(REPO_UPSTREAM_NAME)
-        remote_image = registry.inspect(REPO_UPSTREAM_NAME)
+        registry.pull(REGISTRY_V2_REPO_HELLO_WORLD)
+        remote_image = registry.inspect(REGISTRY_V2_REPO_HELLO_WORLD)
 
         self.assertEqual(local_image[0]["Id"], remote_image[0]["Id"])
-        registry.rmi(REPO_UPSTREAM_NAME)
+        registry.rmi(REGISTRY_V2_REPO_HELLO_WORLD)
 
     def test_pull_image_with_tag(self):
         """Verify that a client can pull the image from Pulp with a tag (on-demand).
@@ -479,15 +446,17 @@ class PullOnDemandContentTestCase(unittest.TestCase):
 
         local_url = (
             urljoin(self.cfg.get_base_url(), self.distribution_with_repo.base_path)
-            + REPO_UPSTREAM_TAG
+            + PULP_HELLO_WORLD_LINUX_TAG
         )
 
         registry.pull(local_url)
         self.teardown_cleanups.append((registry.rmi, local_url))
         local_image = registry.inspect(local_url)
 
-        registry.pull(REPO_UPSTREAM_NAME + REPO_UPSTREAM_TAG)
-        self.teardown_cleanups.append((registry.rmi, REPO_UPSTREAM_NAME + REPO_UPSTREAM_TAG))
-        remote_image = registry.inspect(REPO_UPSTREAM_NAME + REPO_UPSTREAM_TAG)
+        registry.pull(REGISTRY_V2_REPO_HELLO_WORLD + PULP_HELLO_WORLD_LINUX_TAG)
+        self.teardown_cleanups.append(
+            (registry.rmi, REGISTRY_V2_REPO_HELLO_WORLD + PULP_HELLO_WORLD_LINUX_TAG)
+        )
+        remote_image = registry.inspect(REGISTRY_V2_REPO_HELLO_WORLD + PULP_HELLO_WORLD_LINUX_TAG)
 
         self.assertEqual(local_image[0]["Id"], remote_image[0]["Id"])
