@@ -1,4 +1,3 @@
-# coding=utf-8
 """Tests that recursively add container content to repositories."""
 from pulp_smash.pulp3.bindings import (
     delete_orphans,
@@ -19,12 +18,12 @@ from pulpcore.client.pulp_container import (
     ApiException,
     ContainerContainerRemote,
     ContainerContainerRepository,
-    ContentTagsApi,
+    ContainerRepositorySyncURL,
     ContentManifestsApi,
+    ContentTagsApi,
     RemotesContainerApi,
     RepositoriesContainerApi,
     RepositoriesContainerVersionsApi,
-    RepositorySyncURL,
 )
 
 
@@ -51,7 +50,7 @@ class TestManifestCopy(PulpTestCase):
         remote_data = gen_container_remote(upstream_name=PULP_FIXTURE_1)
         cls.remote = cls.remotes_api.create(ContainerContainerRemote(**remote_data))
 
-        sync_data = RepositorySyncURL(remote=cls.remote.pulp_href)
+        sync_data = ContainerRepositorySyncURL(remote=cls.remote.pulp_href)
         sync_response = cls.repositories_api.sync(cls.from_repo.pulp_href, sync_data)
         monitor_task(sync_response.task)
 
@@ -328,7 +327,7 @@ class TestTagCopy(PulpTestCase):
         remote_data = gen_container_remote(upstream_name=PULP_FIXTURE_1)
         cls.remote = cls.remotes_api.create(ContainerContainerRemote(**remote_data))
 
-        sync_data = RepositorySyncURL(remote=cls.remote.pulp_href)
+        sync_data = ContainerRepositorySyncURL(remote=cls.remote.pulp_href)
         sync_response = cls.repositories_api.sync(cls.from_repo.pulp_href, sync_data)
         monitor_task(sync_response.task)
 
@@ -493,7 +492,7 @@ class TestRecursiveAdd(PulpTestCase):
         remote_data = gen_container_remote(upstream_name=PULP_FIXTURE_1)
         cls.remote = cls.remotes_api.create(ContainerContainerRemote(**remote_data))
 
-        sync_data = RepositorySyncURL(remote=cls.remote.pulp_href)
+        sync_data = ContainerRepositorySyncURL(remote=cls.remote.pulp_href)
         sync_response = cls.repositories_api.sync(cls.from_repo.pulp_href, sync_data)
         monitor_task(sync_response.task)
 
