@@ -1,4 +1,3 @@
-# # coding=utf-8
 """Tests that recursively remove container content from repositories."""
 import unittest
 
@@ -23,6 +22,7 @@ from pulpcore.client.pulp_container import (
     ApiException,
     ContainerContainerRemote,
     ContainerContainerRepository,
+    ContainerRepositorySyncURL,
     ContentManifestsApi,
     ContentTagsApi,
     DistributionsContainerApi,
@@ -32,7 +32,6 @@ from pulpcore.client.pulp_container import (
     RepositoriesContainerApi,
     RepositoriesContainerPushApi,
     RepositoriesContainerVersionsApi,
-    RepositorySyncURL,
     TagImage,
 )
 
@@ -59,7 +58,7 @@ class TestRecursiveRemove(PulpTestCase):
         remote_data = gen_container_remote(upstream_name=PULP_FIXTURE_1)
         cls.remote = cls.remotes_api.create(ContainerContainerRemote(**remote_data))
 
-        sync_data = RepositorySyncURL(remote=cls.remote.pulp_href)
+        sync_data = ContainerRepositorySyncURL(remote=cls.remote.pulp_href)
         sync_response = cls.repositories_api.sync(cls.from_repo.pulp_href, sync_data)
         monitor_task(sync_response.task)
 
