@@ -1,10 +1,8 @@
-# coding=utf-8
 """Tests for fetching the list of all repositories."""
 import unittest
 
 from urllib.parse import urljoin
 import requests
-from requests.exceptions import HTTPError
 
 from pulp_smash import api, config
 from pulp_smash.pulp3.bindings import delete_orphans, monitor_task
@@ -25,11 +23,11 @@ from pulpcore.client.pulp_container import (
     ContainerContainerRepository,
     ContainerContainerDistribution,
     ContainerContainerRemote,
+    ContainerRepositorySyncURL,
     DistributionsContainerApi,
     PulpContainerNamespacesApi,
     RepositoriesContainerApi,
     RemotesContainerApi,
-    RepositorySyncURL,
 )
 
 
@@ -53,7 +51,7 @@ class RepositoriesList:
         remote_data = gen_container_remote(upstream_name=PULP_FIXTURE_1)
         cls.remote = cls.remotes_api.create(ContainerContainerRemote(**remote_data))
 
-        sync_data = RepositorySyncURL(remote=cls.remote.pulp_href)
+        sync_data = ContainerRepositorySyncURL(remote=cls.remote.pulp_href)
         sync_response = cls.repositories_api.sync(cls.repository.pulp_href, sync_data)
         monitor_task(sync_response.task)
 
@@ -61,7 +59,7 @@ class RepositoriesList:
         """Fetch repositories from the catalog endpoint."""
         repositories_list_endpoint = urljoin(self.cfg.get_base_url(), "/v2/_catalog")
 
-        with self.assertRaises(HTTPError) as cm:
+        with self.assertRaises(requests.HTTPError) as cm:
             requests.get(repositories_list_endpoint).raise_for_status()
         content_response = cm.exception.response
         authenticate_header = content_response.headers["Www-Authenticate"]
@@ -153,15 +151,15 @@ class RepositoriesListWithPermissionsTestCase(RepositoriesList, unittest.TestCas
 
         cls.user_none = gen_user()
         cls.user_all = gen_user(
-            [
-                "container.pull_containerdistribution",
-                "container.namespace_pull_containerdistribution",
-            ],
+            model_roles=[
+                "container.containerdistribution_consumer",
+                "container.containernamespace_consumer",
+            ]
         )
         cls.user_only_dist1 = gen_user(
-            object_permissions=[
-                ("container.pull_containerdistribution", cls.distribution1.pulp_href),
-                ("container.namespace_pull_containerdistribution", cls.namespace1.pulp_href),
+            object_roles=[
+                ("container.containerdistribution_consumer", cls.distribution1.pulp_href),
+                ("container.containernamespace_consumer", cls.namespace1.pulp_href),
             ]
         )
 
