@@ -1,4 +1,3 @@
-# coding=utf-8
 """Tests for tagging and untagging images."""
 import unittest
 
@@ -22,12 +21,12 @@ from pulpcore.client.pulp_container import (
     ApiException,
     ContainerContainerRemote,
     ContainerContainerRepository,
+    ContainerRepositorySyncURL,
     ContentManifestsApi,
     ContentTagsApi,
     RepositoriesContainerApi,
     RepositoriesContainerPushApi,
     RepositoriesContainerVersionsApi,
-    RepositorySyncURL,
     RemotesContainerApi,
     TagImage,
     UnTagImage,
@@ -81,7 +80,7 @@ class RepositoryTaggingTestCase(TaggingTestCommons, unittest.TestCase):
         remote_data = gen_container_remote(upstream_name=PULP_FIXTURE_1)
         cls.remote = cls.remotes_api.create(ContainerContainerRemote(**remote_data))
 
-        sync_data = RepositorySyncURL(remote=cls.remote.pulp_href)
+        sync_data = ContainerRepositorySyncURL(remote=cls.remote.pulp_href)
         sync_response = cls.repositories_api.sync(cls.repository.pulp_href, sync_data)
         monitor_task(sync_response.task)
 
@@ -157,7 +156,7 @@ class RepositoryTaggingTestCase(TaggingTestCommons, unittest.TestCase):
         self.assertEqual(created_tag.name, "new_tag")
 
         created_tag_manifest = self.manifests_api.read(created_tag.tagged_manifest)
-        self.assertEqual(created_tag_manifest, manifest_b)
+        self.assertEqual(created_tag_manifest.digest, manifest_b.digest)
 
         removed_tag = self.tags_api.list(
             repository_version_removed=new_repository_version_href
@@ -165,7 +164,7 @@ class RepositoryTaggingTestCase(TaggingTestCommons, unittest.TestCase):
         self.assertEqual(removed_tag.name, "new_tag")
 
         removed_tag_manifest = self.manifests_api.read(removed_tag.tagged_manifest)
-        self.assertEqual(removed_tag_manifest, manifest_a)
+        self.assertEqual(removed_tag_manifest.digest, manifest_a.digest)
 
         repository_version = self.versions_api.read(new_repository_version_href)
 
