@@ -1,10 +1,10 @@
-# coding=utf-8
 """Tests for token authentication."""
+import aiohttp
+import asyncio
 import unittest
 
 from urllib.parse import urljoin, urlparse
 import requests
-from requests.exceptions import HTTPError
 
 from pulp_smash import api, config, cli
 from pulp_smash.pulp3.bindings import delete_orphans, monitor_task
@@ -13,12 +13,11 @@ from pulp_smash.pulp3.utils import gen_repo, gen_distribution
 from pulp_container.tests.functional.utils import (
     gen_container_remote,
     gen_container_client,
-    BearerTokenAuth,
-    AuthenticationHeaderQueries,
+    get_auth_for_url,
 )
 from pulp_container.tests.functional.constants import (
     CONTAINER_TAG_PATH,
-    DOCKERHUB_PULP_FIXTURE_1,
+    PULP_FIXTURE_1,
 )
 from pulp_container.constants import MEDIA_TYPE
 
@@ -26,10 +25,10 @@ from pulpcore.client.pulp_container import (
     ContainerContainerDistribution,
     ContainerContainerRepository,
     ContainerContainerRemote,
+    ContainerRepositorySyncURL,
     DistributionsContainerApi,
     RepositoriesContainerApi,
     RemotesContainerApi,
-    RepositorySyncURL,
 )
 
 
@@ -55,10 +54,10 @@ class TokenAuthenticationTestCase(unittest.TestCase):
 
         cls.repository = cls.repositories_api.create(ContainerContainerRepository(**gen_repo()))
 
-        remote_data = gen_container_remote(upstream_name=DOCKERHUB_PULP_FIXTURE_1)
+        remote_data = gen_container_remote(upstream_name=PULP_FIXTURE_1)
         cls.remote = cls.remotes_api.create(ContainerContainerRemote(**remote_data))
 
-        sync_data = RepositorySyncURL(remote=cls.remote.pulp_href)
+        sync_data = ContainerRepositorySyncURL(remote=cls.remote.pulp_href)
         sync_response = cls.repositories_api.sync(cls.repository.pulp_href, sync_data)
         monitor_task(sync_response.task)
 
@@ -88,24 +87,9 @@ class TokenAuthenticationTestCase(unittest.TestCase):
         image_path = "/v2/{}/manifests/{}".format(self.distribution.base_path, "manifest_a")
         latest_image_url = urljoin(self.cfg.get_base_url(), image_path)
 
-        with self.assertRaises(HTTPError) as cm:
-            requests.get(
-                latest_image_url, headers={"Accept": MEDIA_TYPE.MANIFEST_V2}
-            ).raise_for_status()
-
-        content_response = cm.exception.response
-        self.assertEqual(content_response.status_code, 401)
-
-        authenticate_header = content_response.headers["Www-Authenticate"]
-        queries = AuthenticationHeaderQueries(authenticate_header)
+        auth = get_auth_for_url(latest_image_url)
         content_response = requests.get(
-            queries.realm, params={"service": queries.service, "scope": queries.scope}
-        )
-        content_response.raise_for_status()
-        content_response = requests.get(
-            latest_image_url,
-            auth=BearerTokenAuth(content_response.json()["token"]),
-            headers={"Accept": MEDIA_TYPE.MANIFEST_V2},
+            latest_image_url, auth=auth, headers={"Accept": MEDIA_TYPE.MANIFEST_V2}
         )
         content_response.raise_for_status()
         self.compare_config_blob_digests(content_response.json()["config"]["digest"])
@@ -153,3 +137,16 @@ class TokenAuthenticationTestCase(unittest.TestCase):
 
         config_blob_response = self.client.get(manifest_response["config_blob"])
         self.assertEqual(pulled_manifest_digest, config_blob_response["digest"])
+
+
+def test_invalid_user(token_server_url, local_registry):
+    """Test if the token server correctly returns a 401 error in case of invalid credentials."""
+
+    async def get_token():
+        url = f"{token_server_url}?service={local_registry.name}"
+        async with aiohttp.ClientSession() as session:
+            async with session.get(url, auth=aiohttp.BasicAuth("test", "invalid")) as response:
+                return response.status
+
+    status = asyncio.run(get_token())
+    assert status == 401
