@@ -1,5 +1,5 @@
-# coding=utf-8
 """Utilities for tests for the container plugin."""
+import pytest
 import requests
 
 from requests.auth import AuthBase
@@ -27,72 +27,54 @@ from pulpcore.client.pulpcore import (
     ArtifactsApi,
     GroupsApi,
     GroupsUsersApi,
-    TasksApi,
+    UsersApi,
+    UsersRolesApi,
 )
 from pulpcore.client.pulp_container import (
     ApiClient as ContainerApiClient,
+    ContainerRepositorySyncURL,
     ContentBlobsApi,
     ContentManifestsApi,
     ContentTagsApi,
     RemotesContainerApi,
     DistributionsContainerApi,
+    PulpContainerNamespacesApi,
     RepositoriesContainerApi,
     RepositoriesContainerPushApi,
     RepositoriesContainerVersionsApi,
-    RepositorySyncURL,
 )
 
 cfg = config.get_config()
 cli_client = cli.Client(cfg)
 configuration = cfg.get_bindings_config()
 
+core_client = CoreApiClient(configuration)
+users_api = UsersApi(core_client)
+users_roles_api = UsersRolesApi(core_client)
+
 
 TOKEN_AUTH_DISABLED = utils.get_pulp_setting(cli_client, "TOKEN_AUTH_DISABLED")
 
 
-CREATE_USER_CMD = [
-    "from django.contrib.auth import get_user_model",
-    "from django.urls import resolve",
-    "from guardian.shortcuts import assign_perm",
-    "",
-    "user = get_user_model().objects.create(username='{username}')",
-    "user.set_password('{password}')",
-    "user.save()",
-    "",
-    "for permission in {model_permissions!r}:",
-    "    assign_perm(permission, user)",
-    "",
-    "for permission, obj_url in {object_permissions!r}:",
-    "    func, _, kwargs = resolve(obj_url)",
-    "    obj = func.cls.queryset.get(pk=kwargs['pk'])",
-    "    assign_perm(permission, user, obj)",
-]
-
-
-DELETE_USER_CMD = [
-    "from django.contrib.auth import get_user_model",
-    "get_user_model().objects.get(username='{username}').delete()",
-]
-
-
-def gen_user(model_permissions=None, object_permissions=None):
+def gen_user(model_roles=None, object_roles=None):
     """Create a user with a set of permissions in the pulp database."""
-    if model_permissions is None:
-        model_permissions = []
+    if model_roles is None:
+        model_roles = []
 
-    if object_permissions is None:
-        object_permissions = []
+    if object_roles is None:
+        object_roles = []
 
     user = {
         "username": utils.uuid4(),
         "password": utils.uuid4(),
-        "model_permissions": model_permissions,
-        "object_permissions": object_permissions,
     }
-    utils.execute_pulpcore_python(
-        cli_client,
-        "\n".join(CREATE_USER_CMD).format(**user),
-    )
+    new_user = users_api.create(user)
+    user["pulp_href"] = new_user.pulp_href
+
+    for role in model_roles:
+        assign_role_to_user(user, role)
+    for role, content_object in object_roles:
+        assign_role_to_user(user, role, content_object)
 
     api_config = cfg.get_bindings_config()
     api_config.username = user["username"]
@@ -100,47 +82,28 @@ def gen_user(model_permissions=None, object_permissions=None):
     user["core_api_client"] = CoreApiClient(api_config)
     user["groups_api"] = GroupsApi(user["core_api_client"])
     user["group_users_api"] = GroupsUsersApi(user["core_api_client"])
-    user["api_client"] = ContainerApiClient(api_config)
-    user["distribution_api"] = DistributionsContainerApi(user["api_client"])
-    user["remote_api"] = RemotesContainerApi(user["api_client"])
-    user["repository_api"] = RepositoriesContainerApi(user["api_client"])
-    user["pushrepository_api"] = RepositoriesContainerPushApi(user["api_client"])
-    user["repo_version_api"] = RepositoriesContainerVersionsApi(user["api_client"])
-    user["tags_api"] = ContentTagsApi(user["api_client"])
-    user["manifests_api"] = ContentManifestsApi(user["api_client"])
-    user["blobs_api"] = ContentBlobsApi(user["api_client"])
+    user["container_api_client"] = ContainerApiClient(api_config)
+    user["namespace_api"] = PulpContainerNamespacesApi(user["container_api_client"])
+    user["distribution_api"] = DistributionsContainerApi(user["container_api_client"])
+    user["remote_api"] = RemotesContainerApi(user["container_api_client"])
+    user["repository_api"] = RepositoriesContainerApi(user["container_api_client"])
+    user["pushrepository_api"] = RepositoriesContainerPushApi(user["container_api_client"])
+    user["repo_version_api"] = RepositoriesContainerVersionsApi(user["container_api_client"])
+    user["tags_api"] = ContentTagsApi(user["container_api_client"])
+    user["manifests_api"] = ContentManifestsApi(user["container_api_client"])
+    user["blobs_api"] = ContentBlobsApi(user["container_api_client"])
     return user
 
 
 def del_user(user):
     """Delete a user from the pulp database."""
-    utils.execute_pulpcore_python(
-        cli_client,
-        "\n".join(DELETE_USER_CMD).format(**user),
-    )
+    users_api.delete(user["pulp_href"])
 
 
-def add_user_to_distribution_group(user, distribution, group_type, as_user):
-    """Add the user to either owner, collaborator, or consumer group of a distribution."""
-    distribution_pk = distribution.pulp_href.split("/")[-2]
-    collaborator_group = (
-        as_user["groups_api"]
-        .list(name="container.distribution.{}.{}".format(group_type, distribution_pk))
-        .results[0]
-    )
-    as_user["group_users_api"].create(collaborator_group.pulp_href, {"username": user["username"]})
-
-
-def add_user_to_namespace_group(user, namespace_name, group_type, as_user):
-    """Add the user to either owner, collaborator, or consumer group of a namespace."""
-    namespace_collaborator_group = (
-        as_user["groups_api"]
-        .list(name="container.namespace.{}.{}".format(group_type, namespace_name))
-        .results[0]
-    )
-    as_user["group_users_api"].create(
-        namespace_collaborator_group.pulp_href,
-        {"username": user["username"]},
+def assign_role_to_user(user, role, content_object=None):
+    """Assign a role to a user with an optional object."""
+    users_roles_api.create(
+        auth_user_href=user["pulp_href"], user_role={"role": role, "content_object": content_object}
     )
 
 
@@ -200,16 +163,6 @@ def get_container_image_paths(repo, version_href=None):
     ]
 
 
-def gen_container_content_attrs(artifact):
-    """Generate a dict with content unit attributes.
-
-    :param artifact: An artifact.
-    :returns: A semi-random dict for use in creating a content unit.
-    """
-    # FIXME: Add content specific metadata here.
-    return {"_artifact": artifact.pulp_href}
-
-
 def populate_pulp(url=REGISTRY_V2_FEED_URL):
     """Add container contents to Pulp.
 
@@ -222,7 +175,7 @@ def populate_pulp(url=REGISTRY_V2_FEED_URL):
     repositories_api = RepositoriesContainerApi(container_client)
 
     container_remote = remotes_api.create(gen_container_remote(url))
-    sync_data = RepositorySyncURL(remote=container_remote.pulp_href)
+    sync_data = ContainerRepositorySyncURL(remote=container_remote.pulp_href)
     container_repository = repositories_api.create(gen_repo())
     sync_response = repositories_api.sync(container_repository.pulp_href, sync_data)
 
@@ -266,9 +219,6 @@ skip_if = partial(selectors.skip_if, exc=SkipTest)
 identical, except that ``exc`` has been set to ``unittest.SkipTest``.
 """
 
-core_client = CoreApiClient(configuration)
-tasks = TasksApi(core_client)
-
 
 def gen_artifact(url=CONTAINER_IMAGE_URL):
     """Create an artifact."""
@@ -277,3 +227,25 @@ def gen_artifact(url=CONTAINER_IMAGE_URL):
         temp_file.write(response.content)
         artifact = ArtifactsApi(core_client).create(file=temp_file.name)
         return artifact.to_dict()
+
+
+def get_auth_for_url(registry_endpoint_url, auth=None):
+    """Return authentication details based on the the status of token authentication."""
+    if TOKEN_AUTH_DISABLED:
+        auth = ()
+    else:
+        with pytest.raises(requests.HTTPError):
+            response = requests.get(registry_endpoint_url)
+            response.raise_for_status()
+        assert response.status_code == 401
+
+        authenticate_header = response.headers["WWW-Authenticate"]
+        queries = AuthenticationHeaderQueries(authenticate_header)
+        content_response = requests.get(
+            queries.realm, params={"service": queries.service, "scope": queries.scope}, auth=auth
+        )
+        content_response.raise_for_status()
+        token = content_response.json()["token"]
+        auth = BearerTokenAuth(token)
+
+    return auth
