@@ -1,4 +1,3 @@
-# coding=utf-8
 """Tests that verify that images served by Pulp can be pulled."""
 import contextlib
 import unittest
@@ -23,8 +22,8 @@ from pulp_container.tests.functional.constants import (
 from pulpcore.client.pulp_container import (
     ContainerContainerDistribution,
     ContainerContainerRepository,
+    ContainerRepositorySyncURL,
     DistributionsContainerApi,
-    RepositorySyncURL,
     RepositoriesContainerApi,
     RemotesContainerApi,
 )
@@ -77,7 +76,7 @@ class PullContentTestCase(unittest.TestCase):
             cls.teardown_cleanups.append((cls.remotes_api.delete, cls.remote.pulp_href))
 
             # Step 3
-            sync_data = RepositorySyncURL(remote=cls.remote.pulp_href)
+            sync_data = ContainerRepositorySyncURL(remote=cls.remote.pulp_href)
             sync_response = cls.repositories_api.sync(_repo.pulp_href, sync_data)
             monitor_task(sync_response.task)
             cls.repo = cls.repositories_api.read(_repo.pulp_href)
